import os
import sys
import argparse
from AdaptivePELE.utilities import utilities
import matplotlib.pyplot as plt
try:
    # This might fail for older versions of matplotlib (e.g in life cluster)
    plt.style.use("ggplot")
except:
    pass


def parseArguments():
    desc = "Write the information related to the conformation network to file\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("clusteringObject", type=str, help="Path to the clustering object")
    parser.add_argument("suffix", type=str, help="Suffix to append to file names")
    parser.add_argument("metricCol", type=int, help="Column of the metric of interest")
    parser.add_argument("-o", type=str, default=None, help="Output path where to write the files")
    parser.add_argument("-c", "--cond", type=str, default="min", help="Condition on the metric optimality, options are max or min")
    parser.add_argument("-b", "--bindEn", type=int, default=None, help="Column of the binding energy in the report file")
    args = parser.parse_args()
    return args.clusteringObject, args.suffix, args.metricCol, args.o, args.cond, args.bindEn


if __name__ == "__main__":
    clusteringObject, suffix, metricCol, outputPath, metricOptimization, bindingEnergy = parseArguments()
    if outputPath is not None:
        outputPath = os.path.join(outputPath, "")
        if not os.path.exists(outputPath):
            os.makedirs(outputPath)
    else:
        outputPath = ""
    sys.stderr.write("Reading clustering object...\n")
    cl = utilities.readClusteringObject(clusteringObject)
    if cl.conformationNetwork is None:
        sys.exit("Clustering object loaded has no conformation network!!")
    conf = cl.conformationNetwork
    optimalCluster = cl.getOptimalMetric(metricCol, simulationType=metricOptimization)
    pathway = conf.createPathwayToCluster(optimalCluster)
    if not os.path.exists(outputPath+"conformationNetwork%s.edgelist" % suffix):
        sys.stderr.write("Writing conformation network...\n")
        conf.writeConformationNetwork(outputPath+"conformationNetwork%s.edgelist" % suffix)
    if not os.path.exists(outputPath+"FDT%s.edgelist" % suffix):
        sys.stderr.write("Writing FDT...\n")
        conf.writeFDT(outputPath+"FDT%s.edgelist" % suffix)
    if not os.path.exists(outputPath+"pathwayFDT%s.pdb" % suffix):
        sys.stderr.write("Writing pathway to optimal cluster...\n")
        # cl.writePathwayOptimalCluster(outputPath+"pathwayFDT%s.pdb" % suffix)
        cl.writePathwayTrajectory(pathway, outputPath+"pathwayFDT%s.pdb" % suffix)
    if not os.path.exists(outputPath+"nodesPopulation%s.txt" % suffix):
        sys.stderr.write("Writing nodes population...\n")
        cl.writeConformationNodePopulation(outputPath+"nodesPopulation%s.txt" % suffix)
    if not os.path.exists(outputPath+"nodesMetric%s.txt" % suffix):
        sys.stderr.write("Writing nodes metrics...\n")
        cl.writeClusterMetric(outputPath+"nodesMetric%s.txt" % suffix, metricCol)
    if bindingEnergy is not None:
        plt.figure()
        plt.plot(pathway, [cl.clusters.clusters[i].getMetricFromColumn(bindingEnergy) for i in pathway])
        plt.xlabel("Cluster number")
        plt.ylabel("Binding energy(kcal/mol)")
        plt.savefig(outputPath+"bindingEnergy_%s.png" % suffix)
    plt.figure()
    plt.plot(pathway, [cl.clusters.clusters[i].contacts for i in pathway])
    plt.xlabel("Cluster number")
    plt.ylabel("Contacts ratio")
    plt.savefig(outputPath+"contacts_%s.png" % suffix)
    plt.figure()
    plt.plot(pathway, [cl.clusters.clusters[i].getMetricFromColumn(3) for i in pathway])
    plt.xlabel("Cluster number")
    plt.ylabel("Energy(kcal/mol)")
    plt.savefig(outputPath+"totalEnergy_%s.png" % suffix)
    plt.show()



from __future__ import absolute_import, division, print_function, unicode_literals
import math
import os
import numpy
import argparse


def parseArguments():
    desc = "Program that analyses a column of data, printing different statistical values and a histogram if desired."
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("col", type=int, default=1, help="Column with the values (Integer values in [1, N]).")
    parser.add_argument("files", nargs='+', help="Files with the data to analyse.")
    parser.add_argument("-o", metavar="OUTLIERRANGE", type=float, default=1, help="Factor that widens the range for a value not to be considered as an outlier.")
    parser.add_argument("-p", nargs='?', type=int, const=10, metavar='bins', help="Plots histogram. Can take as argument the number of bins.")
    parser.add_argument("-e", action="store_true", default=False, help="If plotting, draw the error bars.")
    args = parser.parse_args()

    return args.col, args.files, args.p, args.o, args.e


def analyseData(data):
    numberOfElements = len(data)
    average = sum(data)
    average /= numberOfElements

    data.sort()
    minVal = data[0]
    maxVal = data[-1]
    median = data[numberOfElements//2]

    variance = 0
    for number in data:
        variance += math.pow(number - average, 2)

    variance = variance/(numberOfElements-1)

    standardDeviation = math.sqrt(variance)

    print("------------------------------")
    print("RESULTS")
    print("------------------------------")
    print("# of elements: ", numberOfElements)
    print("Average value: ", average)
    print("Std deviation: ", standardDeviation)
    print("Maximum vaule: ", maxVal)
    print("Minumum value: ", minVal)
    print("Median  value: ", median)
    print("------------------------------")


def returnListWithoutOutliers(data, outlierRange):
    """
        An outlier is defiend as a datapoint not in [Q1 - 1.5*IQR*outlierRange, Q3 + 1.5*IQR*outlierRange],
        where IQR is the interquartile range: Q3 - Q1
    """
    data.sort()

    dataPointsBefore = len(data)
    Q1 = data[dataPointsBefore//4]
    Q3 = data[3*dataPointsBefore//4]
    IQR = Q3 - Q1

    lowerFence = Q1 - 1.5 * IQR * outlierRange
    upperFence = Q3 + 1.5 * IQR * outlierRange

    filteredData = [i for i in data if i >= lowerFence and i <= upperFence]

    dataPointsAfter = len(filteredData)
    print('Removed ' + str(dataPointsBefore - dataPointsAfter) + ' outliers')

    return filteredData


def printFilenamesAbsolutePath(files):
    print("files:")
    for f in files:
        print(os.path.abspath(f))


def readDataFromFiles(files, column):
    data = []
    for f in files:
        tmpdata = numpy.loadtxt(f, unpack=True, usecols=[column])
        try:
            data.extend(tmpdata)
        except:
            data.append(tmpdata)
    return data


def main():
    column, files, bins, outlierRange, plotErrorBars = parseArguments()
    # We count columns starting with 1
    column -= 1

    data = readDataFromFiles(files, column)

    printFilenamesAbsolutePath(files)
    print("---------------------------")
    print("Before removing outliers:")

    analyseData(data)

    filteredData = returnListWithoutOutliers(data, outlierRange)

    print("---------------------------")
    print("After removing outliers:")

    analyseData(filteredData)

    if bins:
        import histogram
        plotErrorBars = plotErrorBars not in ['False', 'false', 0]
        histogram.plot_histogram(data, bins, plotErrorBars, 1, "Before removing outliers")
        histogram.plot_histogram(filteredData, bins, plotErrorBars, 2, "After removing outliers (%.2f * IQR)" % (outlierRange*1.5))
        # To keep plots "alive"
        raw_input()


if __name__ == '__main__':
    main()



from __future__ import absolute_import, division, print_function, unicode_literals
import os
try:
    import cPickle
except ImportError:
    import pickle as cPickle
import matplotlib.pyplot as plt
import pyemma.msm as msm
import pyemma.plots as mplt
import numpy as np


class MSM:
    """
    def __init__(self, lagtimes, numPCCA, itsOutput=None, numberOfITS=-1,
                 itsErrors=None, error_estimationCK=None, mlags=2, lagtime=None, dtrajs=[]):
    """
    def __init__(self, error=False, dtrajs=None):
        """
            If MSM_object.pkl exists, and we call estimate, does it override whatever was before?
        """

        self.MSMFile = "MSM_object.pkl"
        self.MSM_object = None
        if os.path.exists(self.MSMFile):
            self.MSM_object = self.loadMSM(self.MSMFile)

        self.lagtimes = []
        self.itsOutput = None
        self.numberOfITS = -1
        self.error = error
        self.lagtime = 0
        if dtrajs is None:
            dtrajs = []
        self.dtrajs = dtrajs
        self.stationaryDistributionFilename = "stationaryDistribution.dat"
        self.numPCCA = None

    def estimate(self, lagtime=None, lagtimes=None, numberOfITS=-1):
        self.lagtime = lagtime
        if lagtimes is None:
            lagtimes = []
        self.lagtimes = lagtimes
        self.numberOfITS = numberOfITS
        print("LAGTIME", self.lagtime)
        try:
            self.lagtime = self._calculateITS()  # keep calculating until convergence is reached
        except Exception as err:
            # This error should happen only when running in the MN computing queues
            # with the tkinter matplotlib backend, to avoid it set the default
            # backend to pdf
            if "connect to display" in err.message:
                print("ITS plots not saved because of MN error")
            else:
                raise err
        print("Using lagtime = ", self.lagtime)
        self.buildMSM()
        np.savetxt(self.stationaryDistributionFilename, self.MSM_object.pi)
        self.saveMSM(self.MSM_object)

    def computePCCA(self, numberOfPCCAclusters):
        self.numPCCA = numberOfPCCAclusters
        self.MSM_object.pcca(self.numPCCA)

    def runCKTest(self, mlags):
        self._performCKTest(self.error)

    def getMSM_object(self):
        return self.MSM_object

    def _performCKTest(self, mlags):
        # Chapman-Kolgomorov validation
        nsetsCK = len(self.MSM_object.metastable_sets)
        print(("Performing Chapman-Kolmogorov validation with the %d sets from"
               " the PCCA, when it's done will prompt for the validity of "
               "the model...") % nsetsCK)
        membershipsCK = self.MSM_object.metastable_memberships
        CKObject = self.ChapmanKolmogorovTest(self.MSM_object,
                                              nsetsCK, memberships=membershipsCK,
                                              error_estimation=self.error,
                                              mlags=mlags)
        self.plotChapmanKolmogorovTest(CKObject)
        plt.show()

    def _calculateITS(self):
        is_converged = False
        # its
        print(("Calculating implied time-scales, when it's done will prompt "
               "for confirmation on the validity of the lagtimes..."))
        while not is_converged:
            if not self.error:
                itsErrors = None
            elif self.error:
                itsErrors = "bayes"
            if self.lagtimes and self.lagtimes is not None:
                # workaround to get new its plot at each iteration, the
                # plot_implied_timescales function is calling plt.gca() and
                # recovers the previous plot's axes, by creating a new figure
                # gca gets a set of empty axes and plots are fine
                plt.figure()
                its_object = msm.its(self.dtrajs, lags=self.lagtimes, errors=itsErrors)
                mplt.plot_implied_timescales(its_object, outfile=self.itsOutput, nits=self.numberOfITS)
                plt.savefig("its.png")
            if self.lagtime is not None:
                return self.lagtime
            while True:
                plt.show()
                convergence_answer = raw_input("Has the ITS plot converged?[y/n] ")
                convergence_answer.rstrip()
                convergence_answer = convergence_answer or "y"  # Making yes the default answer
                if convergence_answer.lower() == "y" or convergence_answer.lower() == "yes":
                    is_converged = True
                    lagtime_str = raw_input("Please input the lagtime to construct the MSM: ")
                    lagtime = int(lagtime_str.rstrip())
                    break
                elif convergence_answer.lower() == "n" or convergence_answer.lower() == "no":
                    break
                else:
                    print("Answer not valid. Please answer yes or no")
            if not is_converged:
                new_lagtimes = raw_input("Do you want to define new lagtimes or add to the previous?[add(a)/new(n)] ")
                new_lagtimes.rstrip()
                if new_lagtimes.lower() == "add" or new_lagtimes.lower() == "a":
                    lag_list = raw_input("Please input the lagtimes you want to add separated by a space: ")
                    lag_list.rstrip()
                    self.lagtimes.extend(map(int, lag_list.split(" ")))
                elif new_lagtimes.lower() == "new" or new_lagtimes.lower() == "n":
                    lag_list = raw_input("Please input the new lagtimes separated by a space: ")
                    lag_list.rstrip()
                    self.lagtimes = map(int, lag_list.split(" "))
                self.lagtimes.sort()
        return lagtime

    def writeClustersForWMD(self, outfile="clusters.pdb"):
        tempStr = "HETATM  {:>5d} H1 CLT L {:>4d} {:>8.3f}{:>8.3f}{:>8.3f} 0.75  {:<6f}           H\n"
        with open(outfile, "w") as f:
            for i, coords in enumerate(self.cl.clustercenters):
                f.write(tempStr.format(i+1, i+1, coords[0], coords[1], coords[2], self.MSM_object.stationary_distribution[i]))

    def plotITS(self, its_object, its_plot_file=None, nits=-1):
        its_plot = mplt.plot_implied_timescales(its_object, outfile=its_plot_file, nits=nits)
        plt.savefig("its.eps")
        return its_plot

    def buildMSM(self):
        """ Estimate a MSM from the trajectories using a provided lagtime that
        should be big enough so that the relevant processes have converged.

        self.error: whether to estimate errors or not
        """
        if self.error:
            self.MSM_object = msm.bayesian_markov_model(self.dtrajs, self.lagtime)
        else:
            self.MSM_object = msm.estimate_markov_model(self.dtrajs, self.lagtime)

    def ChapmanKolmogorovTest(self, MSM_object, nsets, memberships=None, error_estimation=False, mlags=2):
        """ Perform the ChapmanKolmogorov test to validate the MSM"""
        return MSM_object.cktest(nsets, memberships=memberships, err_est=error_estimation, mlags=mlags)

    def plotChapmanKolmogorovTest(self, CKObject, layout=None, padding_between=0.1,
                                  padding_top=0.075):
        """ Plot the results of the Chapman-Kolgomorov tests"""
        mplt.plot_cktest(CKObject, layout=layout, padding_between=padding_between,
                         padding_top=padding_top)
        plt.savefig("CK.eps")

    def loadMSM(self, MSMFile):
        with open(MSMFile) as MSMfile:
            MSM_object = cPickle.load(MSMfile)
        return MSM_object

    def saveMSM(self, MSM_object):
        """Save the MSM object to avoid having to run again
        the more computationally expensive part"""
        with open("MSM_object.pkl", "wb") as MSMfile:
            cPickle.dump(MSM_object, MSMfile, 2)



"""
    Program that writes extracted coordinates as pdb files.

    Execution: python generateFiles.py outputDir rawData/traj_*

"""
from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.freeEnergies import computeDeltaG as dg
import sys
import numpy as np
import os


def getNum(trajFilename):
    left = trajFilename.rfind("_")
    return int(trajFilename[left+1:-4])

outputFolder = sys.argv[1]
files = sys.argv[2:]

print("files", files)

for filename in files:
    content = np.loadtxt(filename)

    length = content.shape[0]

    dummyContent = np.ones((length, 1))

    paddedContent = np.delete(np.hstack((content, dummyContent)), 0, axis=1)

    outputFilename = os.path.join(outputFolder, os.path.split(filename)[1])

    dg.writePDB(paddedContent, outputFilename)

    with open(outputFilename, 'r') as f:
        content = "MODEL\n" + "ENDMDL\nMODEL\n".join(f.readlines()) + "ENDMDL\n"
    with open(outputFilename, 'w') as f:
        f.write(content)



import os
import argparse
import schrodinger.structure as st


class OutputFile(object):
    
    def __init__(self, inputfile):
        self.inputfile = inputfile
        self._format = self.inputfile.split(".")[-1]

    
    def get_property(self, Property):
        return self._get_property(Property)


    def set_property(self, Property, value, typedata="r", author="fragpele"):
        self._typedata = typedata
        self._author = author
        return self._set_property(Property, value)



class MaeFile(OutputFile):
    
    
    def __init__(self, inputfile):
        OutputFile.__init__(self, inputfile)
        self._structure = next(st.StructureReader(self.inputfile))
        self._properties = self._structure.property

    def _get_property(self, Property):
        try:
            return self._properties[Property]
        except KeyError:
            raise KeyError("Property {} not found in {}".format(Property, self.inputfile))

    def _set_property(self, Property, value):
        if Property in self._structure.property:
            self._structure.property[Property] = value
        else:
            propertyname = "{}_{}_{}_{}".format(self._typedata, self._author, Property, value)
            self._structure.property[propertyname] =  value
            self._properties = self._structure.property
        return self._properties


def pdb_to_mae(pdb_inputfile, schr_path, mae_output_file=None, remove=False):
    # Get output name from file
    file_info = pdb_inputfile.split("_")
    if not mae_output_file:
        dirpath = os.path.join(os.path.dirname(pdb_inputfile))
        filename = "{}_{}.mae".format(file_info[-3], file_info[-2])
        mae_output_file = os.path.join(dirpath, filename)

    # Get properties from name
    properties = {"BindingEnergy": float(file_info[-1].replace("BindingEnergy", "").replace(".pdb", "")),
                  "epoch": int(file_info[-2].split(".")[0]), 
                  "trajectory": file_info[-2].split(".")[1]
                  }

    # Convert to mae file
    pdb_convert = os.path.join(schr_path, "utilities/pdbconvert")
    os.system("{} -ipdb {} -omae {}".format(pdb_convert, pdb_inputfile, mae_output_file))

    # Set properties to mae_file
    mae_file = MaeFile(mae_output_file)
    for Property, value in properties.items():
        if type(value) == str:
            proper = mae_file.set_property(Property, value, typedata="s")
        if type(value) == int:
            proper = mae_file.set_property(Property, value, typedata="i")
        if type(value) == float:
            proper = mae_file.set_property(Property, value, typedata="r")

    # output maestro file
    with st.StructureWriter(mae_output_file) as writer:
        writer.append(mae_file._structure)

    if remove:
        os.remove(pdb_inputfile)


def add_args(parser):
    parser.add_argument('inputfile', type=str, help="Pdb input file")
    parser.add_argument('--schr', type=str, help="schrodinger root path")
    parser.add_argument('--remove', action="store_true", help="Remove inputfile at exit")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Write mae and sdf files with certain properties')
    add_args(parser)
    args = parser.parse_args()
    pdb_to_mae(args.inputfile, args.schr, remove=args.remove)



from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import glob
import os
import re
import sys
import argparse
import ast


def parseArguments():
    desc = "Program that extends trajectories.\n\
            Trajectories are joined with those from which the spawning cluster was discovered\n\
            Two options are available:\n\
                *) full: Trajectories are traced back to epoch 0\n\
                *) prev: Trajectories are extended with up to the last 'lagtime' snapshots of a previous trajectory\n"

    parser = argparse.ArgumentParser(description=desc, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("--type", choices=['full', 'prev'], default='prev', help="Traj reconstruction type")
    parser.add_argument("-l", "--lagtime", default=0, type=int, help="Lagtime to be used (if proceeds)")
    parser.add_argument("--outputDir", default="allTrajs_reconstructed", help="Output directory")
    parser.add_argument("--inputDir", default="allTrajs", help="Directory with trajectories")
    args = parser.parse_args()

    return args.type, args.lagtime, args.outputDir, args.inputDir


def tryToOpenMapping(mapFilename):
    try:
        with open(mapFilename) as f:
            mapping = f.read().split(":")
            opened = True
    except:
        mapping = None
        opened = False

    return mapping, opened


def sameCoords(coords1, coords2):
    threshold = 1e-4
    diff = np.abs(coords1 - coords2)
    return (diff < threshold).all()


def checkFirstMatchingSnapshot(traj, snapshot, coords):
    for i in range(snapshot, traj.shape[0]):
        if sameCoords(coords[1:], traj[i, 1:]):  # coords[0] is the snapshot num, which is not important and is discarded in MSM
            firstMatchingSnapshot = i
            return firstMatchingSnapshot
    raise IndexError


def findSnapshotAndOpenTraj(trajName, lastSnapshot, coords, firstSnapshot=0):
    if lastSnapshot is None or coords is None:
        return np.loadtxt(trajName)[firstSnapshot:]
    else:
        traj = np.loadtxt(trajName)
        try:
            snapshot = checkFirstMatchingSnapshot(traj, lastSnapshot, coords)
            return traj[firstSnapshot:snapshot]
        except IndexError:
            sys.exit("Did not find matching traj in trajName: %s; coords:%s, from snapshot:%d" % (trajName, coords, lastSnapshot))


def reconstructFullTrajectory(mapping, thisTrajMap, trajNameTempletized, coords):
    """
        thisTrajMap contains the exact point at which a cluster was discovered
        Note that the number of snapshot corresponds to the accepted steps and
        not absolute steps. There are different ways to overcome the limitation.
        The fastest is looking at the report file. A slower way is looking at
        the exact coordinates. It is slower, but the main advantage is that we
        do not need any extra file.
    """
    (epoch, num, snapshot) = thisTrajMap

    try:
        thisTraj = findSnapshotAndOpenTraj(trajNameTempletized % (epoch, num), snapshot, coords)
    except:  # this is due to an error in adaptiveSampling. Once the bug is found, please remove the except block
        epoch += 1
        thisTraj = findSnapshotAndOpenTraj(trajNameTempletized % (epoch, num), snapshot, coords)

    if epoch == 0:
        return thisTraj
    else:
        prevTrajMap = ast.literal_eval(mapping[epoch][num-1])
        return np.vstack((reconstructFullTrajectory(mapping, prevTrajMap, trajNameTempletized, thisTraj[0]), thisTraj))


def addUncountedSnapshots(mapping, thisTrajMap, trajNameTempletized, coords, lagtime):
    """
        This function adds all possible previous uncounted snapshots
        (i.e. those in the last lagtime snapshots) to the current traj

        thisTrajMap contains the exact point at which a cluster was discovered
        Note that the number of snapshot corresponds to the accepted steps and
        not absolute steps. There are different ways to overcome the limitation.
        The fastest is looking at the report file. A slower way is looking at
        the exact coordinates. It is slower, but the main advantage is that we
        do not need any extra file.
    """

    (epoch, num, snapshot) = thisTrajMap
    thisTraj = findSnapshotAndOpenTraj(trajNameTempletized % (epoch, num), snapshot, None)

    if epoch == 0:
        return thisTraj

    prevTrajMap = ast.literal_eval(mapping[epoch][num-1])
    (epoch, num, snapshot) = prevTrajMap
    try:
        # only consider the last "lagtime" snapshots
        # if the initial point was found before the last lagtime snapshots, then: prevTraj = []
        prevTraj = findSnapshotAndOpenTraj(trajNameTempletized % (epoch, num), snapshot, thisTraj[0], firstSnapshot=-lagtime)
    except:
        epoch += 1
        prevTraj = findSnapshotAndOpenTraj(trajNameTempletized % (epoch, num), snapshot, thisTraj[0], firstSnapshot=-lagtime)

    return np.vstack((prevTraj, thisTraj))


def main():
    choice, lagtime, outputDir, inputDir = parseArguments()

    mapFilename = "%d/processorMapping.txt"

    if not os.path.exists(outputDir):
        os.makedirs(outputDir)

    trajWildcard = "traj_%d_*.dat"  # %d for the epoch
    trajName = "traj_%d_%d.dat"  # %d for the epoch and number

    trajNameTempletized = os.path.join(inputDir, trajName)

    allFolders = os.listdir(".")
    epochFolders = [int(re.sub("MSM_", "", epoch)) for epoch in allFolders if epoch.startswith("MSM")]

    numberOfEpochs = max(epochFolders)
    mappings = []
    for epoch in range(0, numberOfEpochs):
        epochMapping, _ = tryToOpenMapping(mapFilename % epoch)
        mappings.append(epochMapping)

    newSizes = []
    for epoch in range(0, numberOfEpochs):
        allFiles = glob.glob(os.path.join(inputDir, trajWildcard % epoch))
        for source in allFiles:
            print(source)
            num = int(source.split("_")[-1][:-4])
            if choice == "full":
                fullTraj = reconstructFullTrajectory(mappings, (epoch, num, None), trajNameTempletized, None)
            elif choice == "prev":
                fullTraj = addUncountedSnapshots(mappings, (epoch, num, None), trajNameTempletized, None, lagtime)

            newSizes.append(fullTraj.shape[0])

            fname = os.path.split(source)[-1]
            dst = os.path.join(outputDir, fname)
            np.savetxt(dst, fullTraj)

    newSizes = np.array(newSizes)
    avgNewSize = np.average(newSizes)
    print("")
    print("Avg new size: %.2f +/- %.2f" % (avgNewSize, np.std(newSizes)))
    try:
        origSize = np.loadtxt(allFiles[0]).shape[0]
        print("Assuming orig trajectories of %d steps" % origSize)
        print("New trajectories are {0:.2f}% larger".format((avgNewSize/origSize - 1)*100))
    except:
        pass

if __name__ == "__main__":
    main()



cdef class Atom:
    cdef public basestring atomSerial, name, resname, resChain, resnum, type, id
    cdef public double x, y, z, mass
    cdef public bint protein

cdef class PDB:
    cdef public dict atoms
    cdef public list atomList, com, centroid
    cdef public double totalMass
    cdef public basestring pdb



# coding: utf-8
from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import os
import argparse
import glob
import re
import numpy as np
import shutil
from AdaptivePELE.atomset import atomset
import sys
#reload(sys)
#sys.setdefaultencoding('utf-8')


class Constants:
    def __init__(self):
        self.extractedTrajectoryFolder = "%s/extractedCoordinates"
        self.baseExtractedTrajectoryName = "coord_"
        self.reportName = '*report_'
        self.outputTrajectoryFolder = "%s/repeatedExtractedCoordinates"
        self.ligandTrajectoryFolder = "ligand_trajs"
        self.ligandTrajectoryBasename = os.path.join(self.ligandTrajectoryFolder, "traj_ligand_%s.pdb")
        self.gatherTrajsFolder = "allTrajs"
        self.gatherTrajsFilename = os.path.join(self.gatherTrajsFolder, "traj_%s_%s.dat")
        self.gatherNonRepeatedFolder = os.path.join(self.gatherTrajsFolder, "extractedCoordinates")
        self.gatherNonRepeatedTrajsFilename = os.path.join(self.gatherNonRepeatedFolder, "traj_%s_%s.dat")


def parseArguments():
    desc = "Program that extracts residue coordinates for a posterior MSM analysis.\
            It either extracts the resname COM coordinates or those of an atomId, depending on the input.\
            It then fills the rejected steps, which is not done by PELE.\
            Finally, trajectories are gathered together in the same allTrajs folder.\
            It automatically detects whether it is an adaptive or a sequential PELE run by looking for folders\
            with numeric names."
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("-f", "--folderWithTrajs", default=".",
                        help="Folder with trajectories (or epochs)")
    # parser.add_argument("-atomId", action=AtomIdAction, help="serial:atomName:resname, e.g. 2048:C1:AIN")
    parser.add_argument("-atomIds", nargs='*', help="serial:atomName:resname, e.g. 2048:C1:AIN. May contain more than one atomId")
    parser.add_argument("-resname", default="", help="Ligand resname")
    parser.add_argument("-CA", "--proteinCA", action="store_true", help="Extract protein alpha carbons coordinates")
    parser.add_argument("-s", "--enforceSequential", action="store_true", help="Force the consideration as sequential run (non-adaptive)")
    parser.add_argument("--setNum", type=int, default=0, help="Sets the number to appear in gathered trajectory in order to avoid clashes between different sequential runs. Ignored in adaptive runs.")
    parser.add_argument("-w", "--writeLigandTrajectory", action="store_true", help="It writes a traj_ligand_XXX.pdb file with the ligand coordinates. The user must delete the original trajectory (if wanted)")
    parser.add_argument("-t", "--totalSteps", type=int, default=0, help="Total number of steps in traj. Equivalent to epoch length in adaptive runs")
    parser.add_argument("-nR", "--noRepeat", action="store_true", help="Flag to avoid repeating the rejected steps")
    # parser.add_argument("-f", nargs='+', help="Files to get coordinates")
    args = parser.parse_args()

    return args.folderWithTrajs, args.atomIds, args.resname, args.proteinCA, args.enforceSequential, args.writeLigandTrajectory, args.totalSteps, args.setNum, args.noRepeat


def isAlphaCarbon(string, writeCA):
    return string[12:16].strip().upper() == "CA" and string[76:80].strip().upper() == "C" and writeCA


def loadAllResnameAtomsInPdb(filename, lig_resname, writeCA):
    fileContent = open(filename).read()
    fileContent = fileContent.split('ENDMDL')
    prunedFileContent = []
    for snapshot in fileContent:
        snapshot = snapshot.split('\n')
        prunedSnapshot = [line for line in snapshot if line[17:20] == lig_resname or isAlphaCarbon(line, writeCA)]
        prunedFileContent.append("\n".join(prunedSnapshot))
    return prunedFileContent


def extractFilenumber(filename):
    last = filename.rfind('.')
    first = filename.rfind('_')
    number = re.sub("[^0-9]", "", filename[first+1:last])
    return number


def getOutputFilename(directory, filename, baseOutputFilename):
    filenumber = extractFilenumber(filename)
    return os.path.join(directory, baseOutputFilename+filenumber+".dat")


def getLigandAlphaCarbonsCoords(allCoordinates, lig_resname):
    trajCoords = []
    for coordinates in allCoordinates:
        PDB = atomset.PDB()
        PDB.initialise(coordinates, resname=lig_resname)
        snapshotCoords = [coord for at in PDB.atomList for coord in PDB.atoms[at].getAtomCoords()]
        PDBCA = atomset.PDB()
        PDBCA.initialise(coordinates, type="PROTEIN")
        snapshotCoords.extend([coord for at in PDBCA.atomList for coord in PDBCA.atoms[at].getAtomCoords()])
        trajCoords.append(snapshotCoords)
    return trajCoords


def getPDBCOM(allCoordinates, lig_resname):
    COMs = []
    for coordinates in allCoordinates:
        pdb = atomset.PDB()
        pdb.initialise(coordinates, resname=lig_resname, heavyAtoms=True)
        COMs.append(pdb.extractCOM())
    return COMs


def getAtomCoord(allCoordinates, lig_resname, atom_Ids):
    coords = []
    # If ever need to speed this up, build a Trajectory class that inherits from PDB
    # and loads the atom according to the position in the snapshot, rather than looking
    # for the atom
    for coordinates in allCoordinates:
        pdb = atomset.PDB()
        pdb.initialise(coordinates, resname=lig_resname, heavyAtoms=True)
        snapshotcoords = []
        for atomId in atom_Ids:
            snapshotcoords.extend(pdb.getAtom(atomId).getAtomCoords())
        coords.append(snapshotcoords)
    return coords


def writeToFile(COMs, outputFilename):
    with open(outputFilename, 'w') as f:
        for i, line in enumerate(COMs):
            f.write(str(i) + ' ')
            for i in range(len(line) - 1):
                f.write(str(line[i]) + ' ')
            f.write(str(line[-1]) + '\n')


def writeFilenameExtractedCoordinates(filename, lig_resname, atom_Ids, pathFolder, writeLigandTrajectory, constants, writeCA):
    allCoordinates = loadAllResnameAtomsInPdb(filename, lig_resname, writeCA)
    if writeLigandTrajectory:
        outputFilename = os.path.join(pathFolder, constants.ligandTrajectoryBasename % extractFilenumber(filename))
        with open(outputFilename, 'w') as f:
            f.write("\nENDMDL\n".join(allCoordinates))

    if writeCA:
        coords = getLigandAlphaCarbonsCoords(allCoordinates[:-1], lig_resname)
    else:
        # because of the way it's split, the last element is empty
        if atom_Ids is None or len(atom_Ids) == 0:
            coords = getPDBCOM(allCoordinates[:-1], lig_resname)
        else:
            coords = getAtomCoord(allCoordinates[:-1], lig_resname, atom_Ids)

    outputFilename = getOutputFilename(constants.extractedTrajectoryFolder, filename,
                                       constants.baseExtractedTrajectoryName)
    writeToFile(coords, outputFilename % pathFolder)


def writeFilenamesExtractedCoordinates(pathFolder, lig_resname, atom_Ids, writeLigandTrajectory, constants, writeCA):
    if not os.path.exists(constants.extractedTrajectoryFolder % pathFolder):
        os.makedirs(constants.extractedTrajectoryFolder % pathFolder)

    originalPDBfiles = glob.glob(pathFolder+'/*traj*.pdb')
    for filename in originalPDBfiles:
        writeFilenameExtractedCoordinates(filename, lig_resname, atom_Ids, pathFolder, writeLigandTrajectory, constants, writeCA)


def parseResname(atom_Ids, lig_resname):
    if atom_Ids is not None and len(atom_Ids) > 0:
        differentResnames = {atomId.split(":")[-1] for atomId in atom_Ids}
        if len(differentResnames) > 1:
            sys.exit("Error! Different resnames provided in atomIds!")
        elif len(differentResnames) == 1:
            extractedResname = differentResnames.pop()

    if (atom_Ids is None or len(atom_Ids) == 0) and lig_resname == "":
        sys.exit("Either resname or atomId should be provided")
    elif lig_resname == "":
        lig_resname = extractedResname  # the atom Id last element is the resname
    elif atom_Ids is not None and len(atom_Ids) > 0:
        if extractedResname != lig_resname:
            sys.exit("Residue name in resname and atomId do not match!")
    return lig_resname


def buildFullTrajectory(steps, trajectory, numtotalSteps, inputTrajectory):
    completeTrajectory = []
    counter = 0
    if len(trajectory) > 0:
        sthWrongInTraj = False
        print(inputTrajectory)
        for i in range(len(trajectory) - 1):
            try:
                repeated = steps[i+1] - steps[i]
            except IndexError:
                print("sth wrong in trajectory %s. This is likely to disagreement between report and trajectory files. Please, fix it manually" % inputTrajectory)
                sthWrongInTraj = True
                break

            for _ in range(repeated):
                snapshot = trajectory[i].split()
                snapshot[0] = str(counter)
                snapshot = ' '.join(snapshot)
                completeTrajectory.append(snapshot)
                counter += 1

        if sthWrongInTraj:
            return completeTrajectory

        if numtotalSteps == 0:
            iterations = list(range(1))
        else:
            iterations = list(range(numtotalSteps + 1 - counter))

        for i in iterations:
            snapshot = trajectory[-1].split()
            snapshot[0] = str(counter)
            snapshot = ' '.join(snapshot)
            completeTrajectory.append(snapshot)
            counter += 1

    return completeTrajectory


def repeatExtractedSnapshotsInTrajectory(inputTrajectory, constants, numtotalSteps):
    extractedTrajFolder, trajFilename = os.path.split(inputTrajectory)
    trajectoryNumber = re.sub(r'\.dat$', '', trajFilename)
    trajectoryNumber = re.sub(constants.baseExtractedTrajectoryName, '', trajectoryNumber)

    origDataFolder = re.sub(constants.extractedTrajectoryFolder % "", "", extractedTrajFolder)
    try:
        reportFile = glob.glob(os.path.join(origDataFolder, constants.reportName + trajectoryNumber))[0]
    except IndexError:
        print("folder", origDataFolder)
        sys.exit("Couldn't find file that matches: %s" % os.path.join(origDataFolder, constants.reportName + trajectoryNumber))

    with open(inputTrajectory) as f:
        trajectory = f.read().splitlines()

    acceptedSteps = np.loadtxt(reportFile, dtype='int', comments='#', usecols=(1,))

    fullTrajectory = buildFullTrajectory(acceptedSteps, trajectory, numtotalSteps, inputTrajectory)

    if len(fullTrajectory) > 0:
        outputFilename = os.path.join(constants.outputTrajectoryFolder % origDataFolder, constants.baseExtractedTrajectoryName + trajectoryNumber + '.dat')
        outputFile = open(outputFilename, 'w')
        for snapshot in fullTrajectory:
            outputFile.write("%s\n" % snapshot)
        outputFile.close()


def repeatExtractedSnapshotsInFolder(folder_name, constants, numtotalSteps):
    inputTrajectoryFolder = constants.extractedTrajectoryFolder % folder_name
    outputTrajectoryFolder = constants.outputTrajectoryFolder % folder_name

    if not os.path.exists(outputTrajectoryFolder):
        os.makedirs(outputTrajectoryFolder)

    inputTrajectories = glob.glob(os.path.join(inputTrajectoryFolder, constants.baseExtractedTrajectoryName + '*'))
    for inputTrajectory in inputTrajectories:
        repeatExtractedSnapshotsInTrajectory(inputTrajectory, constants, numtotalSteps)


def makeGatheredTrajsFolder(constants):
    if not os.path.exists(constants.gatherTrajsFolder):
        os.makedirs(constants.gatherTrajsFolder)
    if not os.path.exists(constants.gatherNonRepeatedFolder):
        os.makedirs(constants.gatherNonRepeatedFolder)


def copyTrajectories(traj_names, destFolderTempletized, folderName):
    for inputTrajectory in traj_names:
        trajectoryNumber = extractFilenumber(os.path.split(inputTrajectory)[1])
        if folderName != ".":  # if not sequential
            setNumber = folderName
        shutil.copyfile(inputTrajectory, destFolderTempletized % (setNumber, trajectoryNumber))


def gatherTrajs(constants, folder_name, setNumber, non_Repeat):
    if non_Repeat:
        trajectoriesFilenames = os.path.join(constants.extractedTrajectoryFolder % folder_name, constants.baseExtractedTrajectoryName + "*")
    else:
        trajectoriesFilenames = os.path.join(constants.outputTrajectoryFolder % folder_name, constants.baseExtractedTrajectoryName + "*")
    trajectories = glob.glob(trajectoriesFilenames)
    copyTrajectories(trajectories, constants.gatherTrajsFilename, folder_name)
    nonRepeatedTrajs = glob.glob(os.path.join(constants.extractedTrajectoryFolder % folder_name, constants.baseExtractedTrajectoryName + "*"))
    copyTrajectories(nonRepeatedTrajs, constants.gatherNonRepeatedTrajsFilename, folder_name)


def main(folder_name=".", atom_Ids="", lig_resname="", numtotalSteps=0, enforceSequential_run=0, writeLigandTrajectory=True, setNumber=0, protein_CA=0, non_Repeat=False):
    constants = Constants()

    lig_resname = parseResname(atom_Ids, lig_resname)

    folderWithTrajs = folder_name

    makeGatheredTrajsFolder(constants)

    # change atomId for list

    if enforceSequential_run:
        folders = ["."]
    else:
        allFolders = os.listdir(folderWithTrajs)
        folders = [epoch for epoch in allFolders if epoch.isdigit()]
        if len(folders) == 0:
            folders = ["."]

    for folder_it in folders:
        pathFolder = os.path.join(folderWithTrajs, folder_it)
        print("Extracting coords from folder %s" % folder_it)
        ligand_trajs_folder = os.path.join(pathFolder, constants.ligandTrajectoryFolder)
        if writeLigandTrajectory and not os.path.exists(ligand_trajs_folder):
            os.makedirs(ligand_trajs_folder)
        writeFilenamesExtractedCoordinates(pathFolder, lig_resname, atom_Ids, writeLigandTrajectory, constants, protein_CA)
        if not non_Repeat:
            print("Repeating snapshots from folder %s" % folder_it)
            repeatExtractedSnapshotsInFolder(pathFolder, constants, numtotalSteps)
        print("Gathering trajs in %s" % constants.gatherTrajsFolder)
        gatherTrajs(constants, folder_it, setNumber, non_Repeat)


if __name__ == "__main__":
    folder, atomIds, resname, proteinCA, enforceSequential, writeLigandTraj, totalSteps, setNum, nonRepeat = parseArguments()
    main(folder, atomIds, resname, totalSteps, enforceSequential, writeLigandTraj, setNum, proteinCA, nonRepeat)



from __future__ import absolute_import, division, print_function, unicode_literals
import AdaptivePELE.atomset.atomset as atomset
import os
import glob
import numpy as np
from AdaptivePELE.utilities import utilities
import argparse
import json


def extendReportWithRmsd(reportFile, rmsds):
    """
        Extend a previous report file with corrected rmsd values

        :param reportFile: Report file to be corrected
        :type reportFile: np.ndarray
        :param rmsds: Rmsd corrected values
        :type rmsds: np.ndarray

        :returns: np.ndarray -- Extended report file with corrected rmsd values
    """
    newShape = reportFile.shape
    fixedReport = np.zeros((newShape[0], newShape[1]+1))
    fixedReport[:, :-1] = reportFile
    fixedReport[:, -1] = rmsds
    return fixedReport


def parseArguments():
    """
        Parse the command-line options

        :returns: object -- Object containing the options passed
    """
    desc = "Program that fixes RMSD symmetries of a PELE report file."\
           "Control file is a JSON file that contains \"resname\", \"native\", "\
           "symmetries, and, optionally, the column to substitute in report. "\
           "Example of content:"\
           "{"\
           "\"resname\" : \"K5Y\","\
           "\"native\" : \"native.pdb\","\
           "\"symmetries\" : {[\"4122:C12:K5Y\":\"4123:C13:K5Y\", \"4120:C10:K5Y\":\"4127:C17:K5Y\"]},"\
           "\"column\" = 5"\
           "}"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("controlFile", type=str, help="Control File name")
    args = parser.parse_args()

    return args.controlFile


def readControlFile(controlFile):
    """
        Extract parameters from controlFile

        :param controlFile: Control file
        :type controlFile: str
        :returns: str, str, list, int -- Name of the ligand in the pdb, filename
            containing the native structure, list of the symmetry groups, column
            corresponding to the rmsd in the report file
    """
    jsonFile = open(controlFile, 'r').read()
    parsedJSON = json.loads(jsonFile)
    resname = parsedJSON["resname"]
    nativeFilename = parsedJSON["native"]
    symmetries = parsedJSON["symmetries"]
    rmsdColInReport = parsedJSON.get("column")
    if not rmsdColInReport:
        # append to the end
        rmsdColInReport = -1

    return resname, nativeFilename, symmetries, rmsdColInReport


def main(controlFile):
    """
        Calculate the corrected rmsd values of conformation taking into account
        molecule symmetries

        :param controlFile: Control file
        :type controlFile: str
    """
    # Constants
    folder = "."
    outputFilename = "fixedReport_%d"
    trajName = "*traj*.pdb"
    reportName = "*report_%d"
    # end constants

    resname, nativeFilename, symmetries, rmsdColInReport = readControlFile(controlFile)

    nativePDB = atomset.PDB()
    nativePDB.initialise(nativeFilename, resname=resname)

    allFolders = os.listdir(folder)
    epochs = [epoch for epoch in allFolders if epoch.isdigit()]

    for epoch in epochs:
        print("Epoch", epoch)
        os.chdir(epoch)
        allTrajs = glob.glob(trajName)

        for traj in allTrajs:
            rmsds = utilities.getRMSD(traj, nativePDB, resname, symmetries)
            trajNum = utilities.getTrajNum(traj)
            try:
                reportFilename = glob.glob(reportName % trajNum)[0]
            except IndexError:
                raise IndexError("File %s not found in folder %s" % (reportName % trajNum, epoch))

            reportFile = np.loadtxt(reportFilename, ndmin=2)

            if rmsdColInReport > 0 and rmsdColInReport < reportFile.shape[1]:
                reportFile[:, rmsdColInReport] = rmsds
                fixedReport = reportFile
            else:
                fixedReport = extendReportWithRmsd(reportFile, rmsds)

            # print(fixedReport)
            np.savetxt(outputFilename % trajNum, fixedReport, fmt=b'%.4f')

        os.chdir("..")

if __name__ == "__main__":
    control_file = parseArguments()
    main(control_file)



from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import numpy as np
from AdaptivePELE.constants import blockNames
from AdaptivePELE.clustering import thresholdcalculatortypes
from abc import abstractmethod


class ThresholdCalculatorBuilder():
    def build(self, clusteringBlock):
        """
            Bulid the selecte thresholdCaulcualtor object

            :param clusteringBlock: Parameters block corresponding to the threshold calculator
            :type clusteringBlock: dict
            :returns: :py:class:`.ThresholdCalculator` -- thresholdCalculator object selected
        """
        try:
            thresholdCalculatorBlock = clusteringBlock[blockNames.ClusteringTypes.thresholdCalculator]
        except KeyError:
            # Default value if no threshold calculator block was defined
            return ThresholdCalculatorHeaviside()

        try:
            typeParam = thresholdCalculatorBlock[blockNames.ThresholdCalculator.type]
        except KeyError:
            sys.exit("Threshold calculator must have a type")

        if typeParam == blockNames.ThresholdCalculator.constant:
            try:
                paramsBlock = thresholdCalculatorBlock[blockNames.ThresholdCalculator.params]
                value = paramsBlock[blockNames.ThresholdCalculatorParams.value]
                return ThresholdCalculatorConstant(value)
            except KeyError:
                print("Using default parameters for constant threshold calculator")
                return ThresholdCalculatorConstant()
        elif typeParam == blockNames.ThresholdCalculator.heaviside:
            try:
                paramsBlock = thresholdCalculatorBlock[blockNames.ThresholdCalculator.params]
                values = paramsBlock[blockNames.ThresholdCalculatorParams.values]
                conditions = paramsBlock[blockNames.ThresholdCalculatorParams.conditions]
                return ThresholdCalculatorHeaviside(conditions, values)
            except KeyError:
                print("Using default parameters for Heaviside threshold calculator")
                return ThresholdCalculatorHeaviside()
        else:
            sys.exit("Unknown threshold calculator type! Choices are: " + str(thresholdcalculatortypes.THRESHOLD_CALCULATOR_TYPE_TO_STRING_DICTIONARY.values()))


class ThresholdCalculator():
    def __init__(self):
        self.type = "BaseClass"  # change for abstract attribute

    @abstractmethod
    def calculate(self, contacts):
        pass

    @abstractmethod
    def __eq__(self, other):
        pass

    def __ne__(self, other):
        return not self.__eq__(other)


class ThresholdCalculatorConstant(ThresholdCalculator):
    def __init__(self, value=2):
        ThresholdCalculator.__init__(self)
        self.type = thresholdcalculatortypes.THRESHOLD_CALCULATOR_TYPES.constant
        self.value = value

    def calculate(self, contacts):
        """
            Calculate the threshold value of a cluster. In this case it is constant,
            the contacts ratio is only passed for compatibility purposes

            :param contacts: Contact ratio
            :type contacts: float
            :returns: float -- threshold value of the cluster
        """
        return self.value

    def __eq__(self, other):
        return self.type == other.type and self.value == other.value

    def getMaxThreshold(self):
        """
            Method that returns the maximum treshold possible, required for new
            distance-ordered clustering(in early development)

            :returns: float -- Maximum threshold possible
        """
        return self.value


class ThresholdCalculatorHeaviside(ThresholdCalculator):
    def __init__(self, conditions=None, values=None):
        ThresholdCalculator.__init__(self)
        self.type = thresholdcalculatortypes.THRESHOLD_CALCULATOR_TYPES.heaviside
        if conditions is None:
            conditions = [1.0, 0.75, 0.5]
        if values is None:
            values = [2, 3, 4, 5.0]

        if len(values) != len(conditions) and len(values) != len(conditions) + 1:
            raise ValueError('The number of values must be equal or one more, than the number of conditions')

        self.conditions = conditions
        self.values = values

    def calculate(self, contacts):
        """
            Calculate the threshold value of a cluster according to the contacts ratio
            and the selected conditions and values

            :param contacts: Contact ratio
            :type contacts: float
            :returns: float -- threshold value of the cluster
        """
        for i in range(len(self.conditions)):
            # change, so that whole condition is in array
            if contacts > self.conditions[i]:
                return self.values[i]
        # the way it's built, it makes more sense to return this value, but, should check that len(value) = len(conditions) + 1 in order to return the "else" value
        return self.values[-1]

    def getMaxThreshold(self):
        """
            Method that returns the maximum treshold possible, required for new
            distance-ordered clustering(in early development)

            :returns: float -- Maximum threshold possible
        """
        return max(self.values)

    def __eq__(self, other):
        return self.type == other.type and\
            np.allclose(self.conditions, other.conditions) and\
            np.allclose(self.values, other.values)



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import socket
import matplotlib
import numpy as np
import os
import argparse
machine = socket.gethostname()
if machine == "bsccv03":
    matplotlib.use('wxagg')
elif 'login' in machine:
    matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
try:
    # This might fail for older versions of matplotlib (e.g in life cluster)
    plt.style.use("ggplot")
except NameError:
    pass


def printHelp():
    """
        Create command line interface

        :returns: str -- Output filename ( if specified )
    """
    desc = "Program that prints the number of clusters of each threshold that"\
           "have been selected by the spwaning throughout an adaptive sampling simulation. "\
           "It must be run in the root folder. "
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("-filename", type=str, default="", help="Output filename")
    parser.add_argument("-o", "--output", type=str, default="", help="Output folder")
    args = parser.parse_args()
    return args.filename, args.output


def main(filename, output_folder):
    print("FILENAME", filename)
    templateSummary = "%d/clustering/summary.txt"
    allFolders = os.listdir(".")
    numberOfEpochs = len([epoch for epoch in allFolders if epoch.isdigit() and
                          os.path.isfile(templateSummary % int(epoch))])

    clustering = np.loadtxt(templateSummary % (numberOfEpochs-1))
    clustersThres = list(set(clustering[:, 4]))
    clustersThres.sort(reverse=True)
    spawningPerThres = np.zeros((numberOfEpochs, len(clustersThres)))
    for i in range(numberOfEpochs):
        clustering = np.loadtxt(templateSummary % i)
        for j, threshold in enumerate(clustersThres):
            spawningPerThres[i, j] = clustering[clustering[:, 4] == threshold, 2].sum()
    line_objects = plt.plot(spawningPerThres)
    plt.legend(line_objects, tuple(["Cluster size %d" % x for x in clustersThres]),
               loc="best")
    plt.title("Processors spawned per epoch and cluster size")
    plt.xlabel("Epoch")
    plt.ylabel("Number of spawned processors")
    if output_folder and not os.path.exists(output_folder):
        os.makedirs(output_folder)
    if filename != "":
        plt.savefig(os.path.join(output_folder, "%s_spawning.png" % filename))
    plt.show()

if __name__ == "__main__":
    filename, out_folder = printHelp()
    main(filename, out_folder)



from __future__ import absolute_import, division, print_function, unicode_literals
import glob
import os
import shutil


class Constants():
    def __init__(self, trajs_path=None):
        self.trajFolder = "allTrajs"
        if trajs_path is not None:
            self.trajFolder = os.path.join(trajs_path, self.trajFolder)
        self.origTrajFiles = os.path.join(self.trajFolder, "traj_*")
        self.trajFileEpoch = os.path.join(self.trajFolder, "traj_%d_*")
        self.nonRepeatedTrajEpoch = os.path.join(self.trajFolder, "extractedCoordinates", "traj_%d_*")
        self.msmFolder = "MSM_%d"
        self.rawDataFolder = os.path.join(self.msmFolder, "rawData")
        self.nonRepeatedCoordsFolder = os.path.join(self.rawDataFolder, "extractedCoordinates")
        self.templetizedControlFileMSM = "templetized_control_MSM.conf"


def extractEpoch(f):
    # Split the filename blablb_0_1.dat into [balblb, 0, 1.dat]
    return f.rsplit("_", 2)[1]


def getAllDifferentEpochs(origTrajFiles):
    trajFiles = glob.glob(origTrajFiles)
    epochs = set([])
    for f in trajFiles:
        epoch = extractEpoch(f)
        epochs.add(int(epoch))
    epochs = sorted(epochs)
    return epochs


def makeMSMFolders(epochs, msmFolder):
    for epoch in epochs:
        if not os.path.exists(msmFolder % epoch):
            os.makedirs(msmFolder % epoch)


def makeRawDataFolders(epochs, rawDataFolder, nonRepeatedCoordsFolder):
    """
        This folder contains symbolic links to the corresponding trajectories
    """
    for epoch in epochs:
        folder = rawDataFolder % epoch
        nonRepeatedFolder = nonRepeatedCoordsFolder % epoch
        if not os.path.exists(folder):
            os.makedirs(folder)
        if not os.path.exists(nonRepeatedFolder):
            os.makedirs(nonRepeatedFolder)


def makeSymbolicLinksForFiles(filelist, destination):
    for src in filelist:
        [srcFolder, filename] = os.path.split(src)
        # srcFolder = os.path.abspath(srcFolder)
        # Switch the symbolic links to use a relative path, so if the
        # folders containing the data are moved they will not break
        srcFolder = os.path.relpath(srcFolder, start=destination)
        src = os.path.join(srcFolder, filename)
        dest = os.path.join(destination, filename)
        try:
            if not os.path.isfile(dest):
                os.symlink(src, dest)
        except OSError:
            pass


def makeSymbolicLinks(epochs, rawDataFolder, trajFileEpoch, trajNonRepeatedEpoch, nonRepeatedRawData):
    for epoch in epochs:
        destFolder = rawDataFolder % epoch
        destNonRepeatedFolder = nonRepeatedRawData % epoch
        for prevEpoch in range(epoch+1):
            sourcesPrevEpoch = glob.glob(trajFileEpoch % prevEpoch)
            makeSymbolicLinksForFiles(sourcesPrevEpoch, destFolder)
            nonRepeatedPrevEpoch = glob.glob(trajNonRepeatedEpoch % prevEpoch)
            makeSymbolicLinksForFiles(nonRepeatedPrevEpoch, destNonRepeatedFolder)


def copyMSMcontrolFile(epochs, msmFolder, templetizedControlFileMSM):
    scriptsFolder = os.path.dirname(os.path.realpath(__file__))
    scriptsFile = os.path.join(scriptsFolder, templetizedControlFileMSM)
    print(scriptsFile)
    for epoch in epochs:
        dst = os.path.join(msmFolder % epoch, templetizedControlFileMSM)
        shutil.copyfile(scriptsFile, dst)


def main(trajsPath=None):
    constants = Constants(trajsPath)

    epochs = getAllDifferentEpochs(constants.origTrajFiles)

    makeMSMFolders(epochs, constants.msmFolder)
    makeRawDataFolders(epochs, constants.rawDataFolder, constants.nonRepeatedCoordsFolder)
    makeSymbolicLinks(epochs, constants.rawDataFolder, constants.trajFileEpoch, constants.nonRepeatedTrajEpoch, constants.nonRepeatedCoordsFolder)
    # copyMSMcontrolFile(epochs, constants.msmFolder, constants.templetizedControlFileMSM)


if __name__ == "__main__":
    main()



class ClusteringTypes:
    type = "type"
    params = "params"
    rmsd = "rmsd"
    contactMap = "contactMap"
    lastSnapshot = "lastSnapshot"
    thresholdCalculator = "thresholdCalculator"
    ligandResname = "ligandResname"
    ligandResnum = "ligandResnum"
    ligandChain = "ligandChain"
    alternativeStructure = "alternativeStructure"
    contactThresholdDistance = "contactThresholdDistance"
    nclusters = "nclusters"
    similarityEvaluator = "similarityEvaluator"
    differenceDistance = "differenceDistance"
    Jaccard = "Jaccard"
    correlation = "correlation"
    symmetries = "symmetries"


class ThresholdCalculator:
    type = "type"
    params = "params"
    heaviside = "heaviside"
    constant = "constant"


class ThresholdCalculatorParams:
    conditions = "conditions"
    values = "values"
    value = "value"


class DensityCalculator:
    type = "type"
    params = "params"
    heaviside = "heaviside"
    null = "null"
    constant = "constant"
    continuous = "continuous"
    exitContinuous = "exitContinuous"


class DensityCalculatorParams:
    conditions = "conditions"
    values = "values"


class StringSpawningTypes:
    type = "type"
    independent = "independent"
    sameWeight = "sameWeight"
    inverselyProportional = "inverselyProportional"
    epsilon = "epsilon"
    fast = "FAST"
    simulatedAnnealing = "simulatedAnnealing"
    # New parameters for variable epsilon(experimental)
    variableEpsilon = "variableEpsilon"
    UCB = "UCB"
    REAP = "REAP"
    null = "null"


class SpawningParams:
    params = "params"
    epsilon = "epsilon"
    temperature = "T"
    threshold = "threshold"
    report_filename = "reportFilename"
    report_col = "metricColumnInReport"
    minValue = "min"
    maxValue = "max"
    condition = "condition"
    # New parameters for variable epsilon(experimental)
    varEpsilonType = "varEpsilonType"
    maxEpsilon = "maxEpsilon"
    minEpsilon = "minEpsilon"
    variationWindow = "variationWindow"  # Last epoch of variable epsilon,if
    # current epoch > than variation Window, set epsilon to minEpsilon
    maxEpsilonWindow = "maxEpsilonWindow"
    period = "period"  # Only useful for periodic epsilon modes
    density = "density"
    metricWeights = "metricWeights"
    linear = "linear"
    boltzmann = "boltzmann"
    alpha = "alpha"
    nclusters = "n"
    metricsInd = "metricsInd"


class SpawningDensity:
    values = "values"
    conditions = "conditions"


class VariableEpsilonTypes:
    linearVariation = "linearVariation"
    contactsVariation = "contactsVariation"


class SimulationType:
    type = "type"
    pele = "pele"
    md = "md"
    test = "test"


class SimulationParams:
    params = "params"
    processors = "processors"
    executable = "executable"
    templetizedControlFile = "controlFile"
    dataFolder = "data"
    documentsFolder = "documents"
    destination = "destination"
    origin = "origin"
    seed = "seed"
    peleSteps = "peleSteps"
    iterations = "iterations"
    exitCondition = "exitCondition"
    metricCol = "metricCol"
    exitValue = "exitValue"
    trajectories = "trajectories"
    modeMovingBox = "modeMovingBox"
    modeMovingBoxBinding = "binding"
    modeMovingBoxUnBinding = "unbinding"
    equilibrationMode = "equilibrationMode"
    equilibrationLastSnapshot = "equilibrationLastSnapshot"
    equilibrationSelect = "equilibrationSelect"
    equilibrationCluster = "equilibrationCluster"
    numberEquilibrationStructures = "numberEquilibrationStructures"
    boxCenter = "boxCenter"
    boxRadius = "boxRadius"
    runEquilibration = "runEquilibration"
    condition = "condition"
    numTrajs = "numberTrajectories"
    equilibrationLength = "equilibrationLength"
    srun = "useSrun"


class ExitConditionType:
    type = "type"
    metric = "metric"
    clustering = "clustering"
    metricMultipleTrajs = "metricMultipleTrajectories"


class ControlFileParams:
    generalParams = "generalParams"
    spawningBlockname = "spawning"
    simulationBlockname = "simulation"
    clusteringBlockname = "clustering"


class GeneralParams:
    restart = "restart"
    outputPath = "outputPath"
    initialStructures = "initialStructures"
    debug = "debug"
    writeAllClustering = "writeAllClusteringStructures"
    nativeStructure = "nativeStructure"



import numpy as np
cimport cython
cimport numpy as np
from libc.math cimport sqrt

@cython.boundscheck(False)
@cython.wraparound(False)
cdef double[:] sum_rows(double[:,:] C):
    cdef Py_ssize_t sx, sy
    cdef double tmp_sum
    sx = C.shape[0]
    sy = C.shape[1]
    cdef double[:] x = np.empty((sy))
    for i in range(sx):
        tmp_sum = 0.0
        for j in range(sx):
            tmp_sum += C[i, j]
        x[i] = tmp_sum

    return x


@cython.boundscheck(False)
@cython.wraparound(False)
@cython.cdivision(True)
def buildRevTransitionMatrix(double[:,:] C):
    """
        #Implemented as Prinz paper
    """
    cdef double[:,:] X, T
    cdef Py_ssize_t sx, sy
    sx = C.shape[0]
    sy = C.shape[1]
    X = np.empty((sx, sy))
    T = np.empty((sx, sy))
    for k in range(sx):
        for ky in range(sy):
            X[k, ky]=C[k,ky]+C[ky,k]
            T[k, ky]= 0
    cdef double[:] x, c
    x = sum_rows(X)
    c = sum_rows(C)
    cdef int iterations = 1000
    cdef int it, i, j
    cdef double a, b, z, tmp, tmp2, tmp3

    for it in range(iterations):
        # if it != 0 and it % 10 == 0: print it
        for i in range(sx):
            X[i,i] = C[i,i] * (x[i] - X[i,i]) / (c[i] - C[i,i])
        x = sum_rows(X)
        for i in range(sx - 1):
            for j in range(i + 1, sx):
                a = c[i] - C[i,j] + c[j] - C[j,i]
                b = c[i]*(x[j]-X[i,j]) + c[j]*(x[i] - X[i,j]) -(C[i,j] + C[j,i])*(x[i] + x[j] - 2*X[i,j])
                z = -(C[i,j] + C[j,i])*(x[i] - X[i,j])*(x[j] - X[i,j])
                tmp = b**2 - 4*a*z
                tmp2 = 2*a
                tmp3 = sqrt(tmp)
                X[j,i] = (-b+tmp3)/tmp2
                X[i,j] = X[j,i]
        x = sum_rows(X)

    for i in range(sx):
        for j in range(sx):
            T[i,j] = X[i,j]/x[i]
    return np.array(T)



from __future__ import unicode_literals
import numpy as np
cimport cython
cimport numpy as np
cimport frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.atomset as atomset


cdef class RMSDCalculator:
    def __init__(self, symmetries=[]):
        """
            :param symmetries: List of dictionaries with gropus of symmetric atoms atomId:symmetricalAtomId corresponding with the symmetrical atoms
            :type symmetries: list
        """
        self.nonSymmetricalAtomsSet = None
        self.symmetries = symmetries

    def __getstate__(self):
        state = {'nonSymmetricalAtomsSet': self.nonSymmetricalAtomsSet,
                 'symmetries': self.symmetries}
        return state

    def __setstate__(self, state):
        self.nonSymmetricalAtomsSet = state['nonSymmetricalAtomsSet']
        self.symmetries = state['symmetries']

    def computeNonSymmAtoms(self, atomset.PDB PDB):
        cdef set allAtomsSet
        allAtomsSet = set(PDB.atoms.keys())
        for group in self.symmetries:
            symmetriesSet = set(group.keys()).union(set(group.values()))
            allAtomsSet -= symmetriesSet
        self.nonSymmetricalAtomsSet = allAtomsSet

    def computeRMSD2(self, atomset.PDB PDB1, atomset.PDB PDB2):
        """
            Compute the squared RMSD between two PDB

            :param PDB1: First PDB with which the RMSD will be calculated
            :type PDB1: PDB
            :param PDB2: First PDB with which the RMSD will be calculated
            :type PDB2: PDB
            :returns: float -- The squared RMSD between two PDB
        """
        cdef double rmsd, d2, d2sm
        cdef basestring atom1Id, atom2Id, atomId
        cdef atomset.Atom atom11, atom12, atom21, atom22, atom1, atom2
        cdef dict group
        cdef int n
        rmsd = 0.0
        if self.nonSymmetricalAtomsSet is None:
            self.computeNonSymmAtoms(PDB1)

        for group in self.symmetries:
            d2 = 0.0
            d2sm = 0.0
            for atom1Id, atom2Id in group.iteritems():
                try:
                    atom11 = PDB1.getAtom(atom1Id)
                    atom12 = PDB1.getAtom(atom2Id)
                    atom21 = PDB2.getAtom(atom1Id)
                    atom22 = PDB2.getAtom(atom2Id)
                except KeyError as err:
                    raise KeyError("Atom %s not found in PDB" % err.message)
                d2 += atom11.squaredDistance(atom21) + atom12.squaredDistance(atom22)
                d2sm += atom12.squaredDistance(atom21) + atom11.squaredDistance(atom22)
            rmsd += min(d2, d2sm)
        for atomId in self.nonSymmetricalAtomsSet:
            try:
                atom1 = PDB1.getAtom(atomId)
                atom2 = PDB2.getAtom(atomId)
            except KeyError as err:
                raise KeyError("Atom %s not found in PDB" % err.message)
            rmsd += atom1.squaredDistance(atom2)
        n = len(PDB1.atoms.items())
        return rmsd/n

    def computeRMSD(self, atomset.PDB PDB1, atomset.PDB PDB2):
        """
            Compute the RMSD between two PDB

            :param PDB1: First PDB with which the RMSD will be calculated
            :type PDB1: PDB
            :param PDB2: First PDB with which the RMSD will be calculated
            :type PDB2: PDB
            :returns: float -- The squared RMSD between two PDB
        """
        return np.sqrt(self.computeRMSD2(PDB1, PDB2))



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import os
import numpy as np
import shutil
import glob
import sys
import matplotlib.pyplot as plt
from AdaptivePELE.freeEnergies import checkDetailedBalance
from AdaptivePELE.freeEnergies import ownBuildMSM
from AdaptivePELE.freeEnergies import computeDeltaG
from AdaptivePELE.freeEnergies import cluster


class Parameters:
    def __init__(self, ntrajs, length, lagtime, nclusters, nruns, useAllTrajInFirstRun, computeDetailedBalance, trajWildcard, folderWithTraj, lagtimes=None, skipFirstSteps=0, clusterCountsThreshold=0, clusteringStride=1):
        # If ntrajs/length = None, all trajs/lengths will be used
        self.trajWildcard = trajWildcard
        self.folderWithTraj = folderWithTraj
        self.ntrajs = ntrajs
        self.length = length
        self.lagtime = lagtime
        self.nclusters = nclusters
        self.nruns = nruns
        self.useAllTrajInFirstRun = useAllTrajInFirstRun
        self.computeDetailedBalance = computeDetailedBalance
        if lagtimes is None:
            self.lagtimes = []
        else:
            self.lagtimes = lagtimes
        self.skipFirstSteps = skipFirstSteps
        self.clusterCountsThreshold = clusterCountsThreshold
        self.clusteringStride = clusteringStride


def __rm(filename):
    try:
        os.remove(filename)
    except OSError:
        pass


def __rmFiles(trajWildcard):
    allfiles = glob.glob(trajWildcard)
    for f in allfiles:
        __rm(f)


def __prepareWorkingControlFile(lagtime, clusters, trajectoryFolder, trajectoryBasename, workingControlFile, lagtimes, clusterCountsThreshold=0, clusteringStride=1):
    """
    #Unused alternative #1, need of a templetized control file
    simulationParameters = simulationrunner.SimulationParameters()
    simulationParameters.templetizedControlFile = controlFile
    sr = simulationrunner.SimulationRunner(simulationParameters)

    controlFileDictionary = {"lagtime": lagtime, "clusters": clusters}
    sr.makeWorkingControlFile(workingControlFile, controlFileDictionary)
    """

    workingFolder = os.path.split(trajectoryFolder)[0]  # note that we assume a workingFolder/origTrajs structure (typically origTrajs=rawData)
    try:
        string = "{\"trajectoryFolder\":\"%s\", \"trajectoryBasename\":\"%s\", \"numClusters\":%d, \"stride\":%d, \"lagtime\":%d, \"itsOutput\":\"its.png\", \"lagtimes\":%s, \"clusterCountsThreshold\":%d}" % (workingFolder, trajectoryBasename, clusters, clusteringStride, lagtime, lagtimes, clusterCountsThreshold)
    except TypeError:
        string = "{\"trajectoryFolder\":\"%s\", \"trajectoryBasename\":\"%s\", \"numClusters\":%d, \"stride\":%d, \"itsOutput\":\"its.png\", \"lagtimes\":%s, \"clusterCountsThreshold\":%d}" % (workingFolder, trajectoryBasename, clusters, clusteringStride, lagtimes, clusterCountsThreshold)
    with open(workingControlFile, 'w') as f:
        f.write(string)


def __constructMSM(workingControlFile):
    ownBuildMSM.main(workingControlFile)


def __computeDG(trajWildcard):
    deltaGLine = computeDeltaG.main(trajWildcard)
    return deltaGLine


def __getDstName(bootstrap, i, trajFile):
    # Equiv lambda
    # getDstName = lambda bootstrap, i, trajFile: "traj_.%d.dat"%i if bootstrap else os.path.split(trajFile)[-1]
    if bootstrap:
        return "traj_.%d.dat" % i
    else:
        return os.path.split(trajFile)[-1]


def copyWorkingTrajectories(fileWildcard, length=None, ntrajs=None, bootstrap=True, skipFirstSteps=0):
    """
        Function that copies trajectories that match "fileWildcard" into the current directory.
        It may copy a subset and a part of them (length)

        Warning! If not using bootstrap, it uses all the trajectories

        :param fileWildcard: Wildcard to match original files
        :type fileWildcard: str
        :param length: Trajectory length to consider, if None (default value), the full trajectory will be considered
        :type length: int
        :param ntrajs: Number of trajs to consider. If None (default value), a number equal to the total  will be considered
        :type ntrajs: int
        :param bootstrap: Bootstrap ntrajs from the original (default is True)
        :type bootstrap: bool
        :param skipFirstSteps: Skip first trajectory steps (default value is 0)
        :type skipFirstSteps: int

        :returns: list -- writenFiles, in order to ease a posterior cleanup

    """
    allFiles = glob.glob(fileWildcard)

    if bootstrap is False:
        trajFiles = allFiles
    else:
        if ntrajs is None:
            ntrajs = len(allFiles)
        trajFiles = np.random.choice(allFiles, ntrajs)

    writenFiles = []
    for i, trajFile in enumerate(trajFiles):
        dst = __getDstName(bootstrap, i, trajFile)
        writenFiles.append(dst)
        traj = np.loadtxt(trajFile)
        if length is None:
            length = -2  # so that later eveything is copied
        try:
            trimmedTraj = traj[skipFirstSteps:length+1, :]
            if len(trimmedTraj) > 0:
                np.savetxt(dst, trimmedTraj, fmt=b"%.4f", delimiter="\t")
        except:
            sys.exit("There is a problem with %s" % trajFile)
    print("Boostraping trajectories", ntrajs, len(trajFiles), len(set(trajFiles)))
    # trajFiles.sort()
    # print(trajFiles)
    return writenFiles


def __cleanupFiles(trajWildcard, cleanupClusterCenters=True):
    __rmFiles("clustering_object.pkl")
    __rmFiles("MSM_object.pkl")
    __rmFiles("discretized/traj_*")
    __rmFiles(trajWildcard)
    if cleanupClusterCenters:
        __rmFiles("discretized/clusterCenter*")


def __setVariablesForFirstIteration(useAllTrajInFirstRun, i, ntrajs):
    if useAllTrajInFirstRun and i == 0:
        print("Not using bootstrap in iteration %d" % i)
        bootstrap = False
        nWorkingTrajs = None  # Not necessary, just to make it explicit that all of them are used
    else:
        print("Using bootstrap in iteration %d" % i)
        bootstrap = True
        nWorkingTrajs = ntrajs

    return bootstrap, nWorkingTrajs


def __copyMSMDataFromRun(i):
    try:  # it may not exist
        shutil.copyfile("its.png", "its_%d.png" % i)
    except IOError:
        pass
    shutil.copyfile("discretized/clusterCenters.dat", "clusterCenters_%d.dat" % i)
    shutil.copyfile("volumeOfClusters.dat", "volumeOfClusters_%d.dat" % i)
    shutil.copyfile("clusters.pdb", "clusters_%d.pdb" % i)
    shutil.copyfile("pmf_xyzg.dat", "pmf_xyzg_%d.dat" % i)
    shutil.copyfile("MSM_object.pkl", "MSM_object_%d.pkl" % i)
    if i == 0:
        try:
            shutil.copyfile("db_frobenius.eps", "db_frobenius_%d.eps" % i)
            shutil.copyfile("db_abs_diff.eps", "db_abs_diff_%d.eps" % i)
            shutil.copyfile("db_flux.eps", "db_flux_%d.eps" % i)
        except IOError:
            pass


def __printList(l, label):
    print(label)
    print("=====")
    for el in l:
        print(el)


def __getMeanAndStdFromList(l, accessFunction=lambda x: x):
    values = [float(accessFunction(element)) for element in l]
    return np.mean(values), np.std(values)


def getRepresentativePDBs(filesWildcard, run):
    files = glob.glob(filesWildcard)
    trajs = [np.loadtxt(f)[:, 1:] for f in files]
    cl = cluster.Cluster(0, "", "")
    cl.clusterCenters = np.loadtxt(cl.clusterCentersFile)
    dtrajs = cl.assignNewTrajectories(trajs)
    numClusters = cl.clusterCenters.shape[0]
    centersInfo = {x: {"structure": None, "minDist": 1e6} for x in range(numClusters)}
    for i, traj in enumerate(trajs):
        traj_name = files[i]
        _, epochNum, trajNum = os.path.splitext(traj_name)[0].split("_", 2)
        for nSnap, snapshot in enumerate(traj):
            clusterInd = dtrajs[i][nSnap]
            dist = np.linalg.norm(cl.clusterCenters[clusterInd]-snapshot)
            if dist < centersInfo[clusterInd]['minDist']:
                centersInfo[clusterInd]["minDist"] = dist
                centersInfo[clusterInd]["structure"] = (epochNum, trajNum, str(nSnap))

    if not os.path.exists("representative_structures"):
        os.makedirs("representative_structures")
    with open("representative_structures/representative_structures_%d.dat" % run, "w") as fw:
        fw.write("Cluster\tEpoch\tTrajectory\tSnapshot\n")
        for clNum in range(numClusters):
            fw.write("%d\t" % clNum+"\t".join(centersInfo[clNum]["structure"])+"\n")


def estimateDG(parameters, cleanupClusterCentersAtStart=False):
    """
        Estimates the absolute binding free energy using the parameters in the Parameters object.

        It copies the trajectory files from "folderWithTraj" into the current folder makes a certain number of iterations

        Documentation needs to be expanded, but the code style aims to help readability
    """

    workingControlFile = "control_MSM.conf"
    origFilesWildcard = os.path.join(parameters.folderWithTraj, parameters.trajWildcard)
    origFilesNonRepeatedWildcard = os.path.join(parameters.folderWithTraj, "extractedCoordinates", parameters.trajWildcard)

    __prepareWorkingControlFile(parameters.lagtime, parameters.nclusters, parameters.folderWithTraj, parameters.trajWildcard, workingControlFile, parameters.lagtimes, parameters.clusterCountsThreshold, parameters.clusteringStride)

    deltaGs = []
    detailedBalance = []
    __cleanupFiles(parameters.trajWildcard, cleanupClusterCentersAtStart)

    for i in range(parameters.nruns):
        bootstrap, nWorkingTrajs = __setVariablesForFirstIteration(parameters.useAllTrajInFirstRun, i, parameters.ntrajs)

        copyWorkingTrajectories(origFilesWildcard, parameters.length, nWorkingTrajs, bootstrap, parameters.skipFirstSteps)

        __constructMSM(workingControlFile)

        deltaG = __computeDG(parameters.trajWildcard)
        deltaGs.append(deltaG)

        getRepresentativePDBs(origFilesNonRepeatedWildcard, i)

        if parameters.computeDetailedBalance:
            avgAsymmetricFlux = checkDetailedBalance.main(folder="discretized", countsThreshold=0, lagtime=parameters.lagtime, printFigs=False)
            detailedBalance.append(avgAsymmetricFlux)

        __copyMSMDataFromRun(i)

        __cleanupFiles(parameters.trajWildcard, True)

        # Close all open matplotlib windows, apparently the its plot opens a lot
        # of windows that are not closed, which consumes a lot of memory (not
        # sure how much exactly)
        plt.close("all")
    # PLOT RESULTS
    # FIX TO WORK WITH NONES
    # print("clusters: %d, ntrajs: %d, trajLength: %d, lagtime: # % d"%(parameters.nclusters, parameters.ntrajs, parameters.length, # parameters.lagtime))
    __printList(deltaGs, "dG")
    meanDG, stdDG = __getMeanAndStdFromList(deltaGs, lambda element: element.split()[1])
    print("dG = %f +- %f" % (meanDG, stdDG))
    __printList(detailedBalance, "Asymmetric fluxes (see D.Lecina PhD thesis for more info)")
    meanDB, stdDB = __getMeanAndStdFromList(detailedBalance)  # DB from detailed balance
    print("Asymmetric flux = %f +- %f" % (meanDB, stdDB))
    with open("results_summary.txt", "w") as fw:
        fw.write("Lagtime %d\n" % parameters.lagtime)
        fw.write("Number of clusters %d\n" % parameters.nclusters)
        fw.write("\n")
        fw.write("=====\n")
        fw.write("dG\n")
        fw.write("bound    Delta G     Delta W     Binding Volume:     Binding Volume contribution\n")
        for el in deltaGs:
            fw.write("%s\n" % el)
        fw.write("=====\n")
        fw.write("Asymmetric fluxes (see D.Lecina PhD thesis for more info)\n")
        for el in detailedBalance:
            fw.write("%.3f\n" % el)
        fw.write("\n")
        fw.write("dG = %f +- %f\n" % (meanDG, stdDG))
        fw.write("Asymmetric flux = %f +- %f\n" % (meanDB, stdDB))

    return meanDG, stdDG, meanDB, stdDB

if __name__ == "__main__":
    params = Parameters(ntrajs=None, length=None, lagtime=25, nclusters=100,
                        nruns=1, skipFirstSteps=0, useAllTrajInFirstRun=True,
                        computeDetailedBalance=True, trajWildcard="traj_*",
                        folderWithTraj="rawData", lagtimes=[1, 10, 25, 50],
                        clusterCountsThreshold=0, clusteringStride=1)
    estimateDG(params, cleanupClusterCentersAtStart=False)



from __future__ import absolute_import, division, print_function, unicode_literals
import unittest
import pickle
import shutil
import os
import AdaptivePELE.adaptiveSampling as adaptiveSampling
import AdaptivePELE.atomset.atomset as atomset
from AdaptivePELE.clustering import clustering
import socket


class TestadaptiveSampling(unittest.TestCase):

    def checkClusteringObjects(self, goldenClusters, outputPath):
        # goldenPathObject = os.path.join(goldenPath, "%d/clustering/object.pkl")
        outputPathObject = os.path.join(outputPath, "%d/clustering/object.pkl")
        for i in range(2, 3):
            with open(outputPathObject % i, 'rb') as f2:
                outputCluster = pickle.load(f2)
            # with open(goldenPathObject % i, 'rb') as f:
            #     goldenCluster = pickle.load(f)
            # outputCluster.clusters.printClusters()
            self.assertEqual(outputCluster.clusters.getNumberClusters(), len(goldenClusters))
            for goldCl, outCl in zip(goldenClusters, outputCluster.clusters.clusters):
                self.assertEqual(outCl, goldCl)

    def checkStartingConformations(self, goldenPath, outputPath):
        goldenPathInitial = os.path.join(goldenPath, "%d/initial_%d.pdb")
        outputPathInitial = os.path.join(outputPath, "initial_%d_%d.pdb")
        for j in range(3):
            for ij in range(1, 5):
                goldenInitial = atomset.PDB()
                goldenInitial.initialise(goldenPathInitial % (j, ij))

                outputInitial = atomset.PDB()
                if j == 0:
                    # initial structures will always be "initial_0_0.pdb"
                    outputInitial.initialise(outputPathInitial % (j, 0))
                else:
                    outputInitial.initialise(outputPathInitial % (j, ij % 4))
                self.assertEqual(goldenInitial, outputInitial)

    def checkTrajectories(self, goldenPath, outputPath):
        goldenPathTrajectory = os.path.join(goldenPath, "%d/trajectory_%d.pdb")
        outputPathTrajectory = os.path.join(outputPath, "%d/trajectory_%d.pdb")

        for epoch in range(3):
            for i in range(1, 5):
                goldenTrajFile = open(goldenPathTrajectory % (epoch, i), 'r')
                goldenTraj = goldenTrajFile.read()

                goldenTrajFile.close()
                outputTrajFile = open(outputPathTrajectory % (epoch, i), 'r')
                outputTraj = outputTrajFile.read()
                outputTrajFile.close()
                self.assertEqual(goldenTraj, outputTraj)

    def integrationTest(self, controlFile, goldenPath, outputPath, goldenClusters):
        # Function to test --> integration test
        adaptiveSampling.main(controlFile)

        # Assertions
        tmpFolder = "tmp_" + outputPath.replace("/", "_")

        self.checkClusteringObjects(goldenClusters, outputPath)
        self.checkStartingConformations(goldenPath, tmpFolder)
        self.checkTrajectories(goldenPath, outputPath)

        # cleanup
        shutil.rmtree(outputPath)
        shutil.rmtree(tmpFolder)

    def testIntegration1(self):
        """
            Simulations are not run, trajectories and reports are copied
        """
        controlFile = "tests/data/3ptb_data/integrationTest1.conf"
        goldenPath = "tests/data/3ptb_data/originTest1"
        outputPath = "tests/data/3ptb_data/Test1"
        elements = [28, 17, 5, 18, 1, 3]
        goldenClusters = []
        metrics = [
                [ 1.00000e+00, 0.00000e+00, 0.00000e+00, -7.49807e+03, 2.01909e+01, 2.16436e-01],
                [ 1.00000e+00, 0.00000e+00, 0.00000e+00, -7.49806e+03, 1.85232e+01, 2.29384e-01],
                [ 1.00000e+00, 0.00000e+00, 0.00000e+00, -7.49801e+03, 1.82444e+01, 2.53929e-01],
                [ 1.00000e+00, 0.00000e+00, 0.00000e+00, -7.49800e+03, 2.24539e+01, 2.66941e-01],
                [ 1.00000e+00, 5.00000e+00, 5.00000e+00, -7.49797e+03, 2.23322e+01, 3.08604e-01],
                [ 1.00000e+00, 3.00000e+00, 3.00000e+00, -7.49829e+03, 1.48606e+01, 7.32479e-03]
                ]
        for i in range(6):
            pdb = atomset.PDB()
            pdb.initialise(goldenPath+"/2/clustering/cluster_%d.pdb" % i, resname="AEN")
            cluster = clustering.Cluster(pdb, thresholdRadius=4, metrics=metrics[i])
            cluster.elements = elements[i]
            cluster.contacts = 0
            goldenClusters.append(cluster)
        self.integrationTest(controlFile, goldenPath, outputPath, goldenClusters)

    def testIntegration2(self):
        """
            Simulations are not run, trajectories and reports are copied
        """
        controlFile = "tests/data/3ptb_data/integrationTest2.conf"
        goldenPath = "tests/data/3ptb_data/srcTest2Epsilon"
        outputPath = "tests/data/3ptb_data/Test2"
        elements = [35, 14, 8, 14, 1]
        metrics = [[1.0, 0.0, 0.0, -7498.07, 20.1909, 0.216436],
                   [1.0, 0.0, 0.0, -7498.06, 18.5232, 0.229384],
                   [1.0, 0.0, 0.0, -7498.01, 18.6059, 0.253929],
                   [1.0, 0.0, 0.0, -7498.05, 22.4539, 0.238184],
                   [1.0, 4.0, 4.0, -7498.05, 22.7766, 0.242335]]

        goldenClusters = []
        for i in range(5):
            pdb = atomset.PDB()
            pdb.initialise(goldenPath+"/2/clustering/cluster_%d.pdb" % i, resname="AEN")
            cluster = clustering.Cluster(pdb, 4, metrics=metrics[i])
            cluster.elements = elements[i]
            cluster.contacts = 0
            goldenClusters.append(cluster)

        self.integrationTest(controlFile, goldenPath, outputPath, goldenClusters)

    def testIntegration3(self):
        """
            Simulations are actually run
        """
        controlFile = "tests/data/3ptb_data/integrationTest3.conf"
        goldenPath = "tests/data/3ptb_data/originTest3"
        outputPath = "tests/data/3ptb_data/Test3"
        elements = [22, 14, 16, 19, 1]
        goldenClusters = []
        metrics = [
                [1.00000e+00, 0.00000e+00, 0.00000e+00, -5.28675e+02, 6.41969e-03],
                [1.00000e+00, 0.00000e+00, 0.00000e+00, -5.28657e+02, 1.88599e-02],
                [1.00000e+00, 0.00000e+00, 0.00000e+00, -5.28675e+02, 6.41969e-03],
                [1.00000e+00, 0.00000e+00, 0.00000e+00, -5.28665e+02, 1.24213e-02],
                [1.00000e+00, 5.00000e+00, 5.00000e+00, -5.28660e+02, 1.75149e-02]
                ]
        for i in range(5):
            pdb = atomset.PDB()
            pdb.initialise(goldenPath+"/2/clustering/cluster_%d.pdb" % i, resname="AEN")
            cluster = clustering.Cluster(pdb, 4, metrics=metrics[i])
            cluster.elements = elements[i]
            cluster.contacts = 0
            goldenClusters.append(cluster)
        name = socket.gethostname()
        if "bsccv" not in name and "login" not in name:
            print("Some integration can't be run due to not having PELE  installed")
            return True
        self.integrationTest(controlFile, goldenPath, outputPath, goldenClusters)

    def testRestartEmptyClustering(self):
        controlFile = "tests/data/3ptb_data/restartTest.conf"
        outputPath = "tests/data/3ptb_data/RestartTest"
        tmpFolder = "tmp_" + outputPath.replace("/", "_")
        clusteringObjectPath = os.path.join(outputPath, "2", "clustering", "object.pkl")
        if not os.path.exists(os.path.join(outputPath, "1", "clustering")):
            os.makedirs(os.path.join(outputPath, "1", "clustering"))
        name = socket.gethostname()
        if "bsccv" not in name and "login" not in name:
            print("Some integration can't be run due to not having PELE  installed")
            return True
        # Function to test --> integration test
        shutil.copy("tests/data/3ptb_data/object_test_bk.pkl", os.path.join(outputPath, "1", "clustering", "object.pkl"))
        adaptiveSampling.main(controlFile)
        # Assertions
        self.assertTrue(adaptiveSampling.checkIntegrityClusteringObject(clusteringObjectPath))
        # Remove clustering object from the simulation and create and empty one
        open(clusteringObjectPath, "w").close()
        # cleanup
        shutil.rmtree(tmpFolder)



from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.utilities import utilities
import argparse
import os


def parseArguments():
    desc = "Program that writes a trajectory into separate pdbs."
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("files", nargs='+', help="Trajectory files to split")
    parser.add_argument("-o", type=str, default=".", help="Output dir")
    args = parser.parse_args()
    return args.files, args.o


files, outputDir = parseArguments()
utilities.makeFolder(outputDir)

for f in files:
    name = os.path.split(f)[-1]
    templateName = os.path.join(outputDir, name[:-4] + "_%d.pdb")
    snapshots = utilities.getSnapshots(f)
    print(len(snapshots))
    for i, snapshot in enumerate(snapshots):
        print(templateName % i)
        with open(templateName % i, 'w') as of:
            of.write(snapshot)



"""
    Recreate the trajectory fragments to the led to the discovery of a snapshot,
    specified by the tuple (epoch, trajectory, snapshot) and write as a pdb file
"""
from __future__ import print_function
import os
import sys
import argparse
import glob
import itertools
from AdaptivePELE.utilities import utilities


def parseArguments():
    """
        Parse the command-line options

        :returns: :py:class:`.Clustering`, int, int, int, str -- Clustering
            object, number of trajectory, number of snapshot, number of epoch,
            output path where to write the files
    """
    desc = "Write the information related to the conformation network to file\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("epoch", type=str, help="Path to the epoch to search the snapshot")
    parser.add_argument("trajectory", type=int, help="Trajectory number")
    parser.add_argument("snapshot", type=int, help="Snapshot to select (in accepted steps)")
    parser.add_argument("-o", type=str, default=None, help="Output path where to write the files")
    parser.add_argument("--name", type=str, default="pathway.pdb", help="Name of the pdb to write the files")
    args = parser.parse_args()
    return args.trajectory, args.snapshot, args.epoch, args.o, args.name


def main(trajectory, snapshot, epoch, outputPath, out_filename):
    if outputPath is not None:
        outputPath = os.path.join(outputPath, "")
        if not os.path.exists(outputPath):
            os.makedirs(outputPath)
    else:
        outputPath = ""
    if os.path.exists(outputPath+out_filename):
        # If the specified name exists, append a number to distinguish the files
        name, ext = os.path.splitext(out_filename)
        out_filename = "".join([name, "_%d", ext])
        i = 1
        while os.path.exists(outputPath+out_filename % i):
            i += 1
        out_filename %= i
    pathway = []
    # Strip out trailing backslash if present
    pathPrefix, epoch = os.path.split(epoch.rstrip("/"))
    sys.stderr.write("Creating pathway...\n")
    while True:
        filename = glob.glob(os.path.join(pathPrefix, epoch, "*traj*_%d.pdb" % trajectory))
        snapshots = utilities.getSnapshots(filename[0])
        snapshots = snapshots[:snapshot+1]
        pathway.insert(0, snapshots)
        if epoch == '0':
            # Once we get to epoch 0, we just need to append the trajectory
            # where the cluster was found and we can break out of the loop
            break
        procMapping = open(os.path.join(pathPrefix, epoch, "processorMapping.txt")).read().rstrip().split(':')
        epoch, trajectory, snapshot = map(int, procMapping[trajectory-1][1:-1].split(','))
        epoch = str(epoch)
    sys.stderr.write("Writing pathway...\n")
    with open(outputPath+out_filename, "a") as f:
        f.write("ENDMDL\n".join(itertools.chain.from_iterable(pathway)))


if __name__ == "__main__":
    trajectory, snapshot, epoch, outputPath, out_filename = parseArguments()
    main(trajectory, snapshot, epoch, outputPath, out_filename)



from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.spawning import densitycalculator
from AdaptivePELE.spawning import densitycalculatortypes
import unittest


class densityCalculatorTest(unittest.TestCase):
    def testDensityCalculatorUndefinedBlock(self):
        spawningBlock = {}
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)
        self.assertEqual(densityCalculator.type, densitycalculatortypes.DENSITY_CALCULATOR_TYPES.null)

    def testDensityCalculatorNullCalculator(self):
        spawningBlock = {
            type: "irrelevant",
            "density": {
                "type": "null"
            }
        }
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)
        self.assertEqual(densityCalculator.type, densitycalculatortypes.DENSITY_CALCULATOR_TYPES.null)

    def testDensityCalculatorHeavisideNoParams(self):
        spawningBlock = {
            type: "irrelevant",
            "density": {
                "type": "heaviside"
            }
        }
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)

        goldenValues = [1.]
        goldenConditions = []

        self.assertEqual(densityCalculator.type, densitycalculatortypes.DENSITY_CALCULATOR_TYPES.heaviside)
        self.assertEqual(densityCalculator.values, goldenValues)
        self.assertEqual(densityCalculator.conditions, goldenConditions)

    def testDensityCalculatorHeavisideParams(self):
        spawningBlock = {
            type: "irrelevant",
            "density": {
                "type": "heaviside",
                "params": {
                    "conditions": [1, 2],
                    "values": [1, 2]
                }
            }
        }
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)

        goldenValues = [1., 2.]
        goldenConditions = [1., 2.]

        self.assertEqual(densityCalculator.type, densitycalculatortypes.DENSITY_CALCULATOR_TYPES.heaviside)
        self.assertEqual(densityCalculator.values, goldenValues)
        self.assertEqual(densityCalculator.conditions, goldenConditions)

    def testDensityCalculatorContinuousParams(self):
        spawningBlock = {
            type: "irrelevant",
            "density": {
                "type": "continuous"
            }
        }
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)

        self.assertAlmostEqual(densityCalculator.calculate(0.5, 8), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 8), 8)
        self.assertAlmostEqual(densityCalculator.calculate(0.5, 6), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 6), 8)
        self.assertAlmostEqual(densityCalculator.calculate(0.5, 4), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 4), 8)
        # Changed behaviour, now densities are always set with contact threshold
        # 8
        # self.assertAlmostEqual(densityCalculator.calculate(0.2, 6), 1)
        # self.assertAlmostEqual(densityCalculator.calculate(0.5, 6), 8)
        # self.assertAlmostEqual(densityCalculator.calculate(0.05, 4), 1)
        # self.assertAlmostEqual(densityCalculator.calculate(0.2, 4), 8)

    def testDensityCalculatorInverseContinuousParams(self):
        spawningBlock = {
            type: "irrelevant",
            "density": {
                "type": "exitContinuous"
            }
        }
        densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
        densityCalculator = densityCalculatorBuilder.build(spawningBlock)

        self.assertAlmostEqual(densityCalculator.calculate(0.5, 8), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 8), 0.125)
        self.assertAlmostEqual(densityCalculator.calculate(0.0, 8), 3.375)
        self.assertAlmostEqual(densityCalculator.calculate(0.5, 6), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 6), 0.125)
        self.assertAlmostEqual(densityCalculator.calculate(0.5, 4), 1)
        self.assertAlmostEqual(densityCalculator.calculate(1.5, 4), 0.125)



from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import os
import glob


templetizedEpochFolder = "%d"


def findTrajectoryFirstBindingEvent(metrics, thresholdValue, unBinding=False):
    if unBinding:
        firstBindingEvent = np.argmax(metrics[:, 1] >= thresholdValue)
        if metrics[firstBindingEvent, 1] >= thresholdValue:
            return metrics[firstBindingEvent, 0]
        else:
            return None
    else:
        # argmax returns "0" if no argument matches
        firstBindingEvent = np.argmax(metrics[:, 1] <= thresholdValue)
        if metrics[firstBindingEvent, 1] <= thresholdValue:
            return metrics[firstBindingEvent, 0]
        else:
            return None


def findEpochFirstBindingEvent(thresholdValue, columnInReport, reportWildcard="*report_*", unBinding=False):
    foundBindingEvent = False
    minNumberOfSteps = 1e10

    reportFiles = glob.glob(reportWildcard)
    for reportFile in reportFiles:
        metrics = np.loadtxt(reportFile, usecols=(1, columnInReport), ndmin=2)
        if metrics.shape[0] == 0:
            continue
        firstBindingEvent = findTrajectoryFirstBindingEvent(metrics, thresholdValue, unBinding=unBinding)
        if firstBindingEvent is not None:
            foundBindingEvent = True
            if firstBindingEvent < minNumberOfSteps:
                minNumberOfSteps = firstBindingEvent
    if foundBindingEvent:
        return minNumberOfSteps
    else:
        return None


def getAllSortedEpochs():
    allFolders = os.listdir(".")
    epochFolders = sorted([int(epoch) for epoch in allFolders if epoch.isdigit()])
    return epochFolders


def findFirstBindingEvent(stepsPerEpoch, columnInReport, thresholdValue, unBinding=False):
    epochFolders = getAllSortedEpochs()

    for epoch in epochFolders:
        os.chdir(str(epoch))
        epochFirstBindingEvent = findEpochFirstBindingEvent(thresholdValue, columnInReport, unBinding=unBinding)

        if epochFirstBindingEvent is not None:
            return epochFirstBindingEvent + epoch * stepsPerEpoch

        os.chdir("..")

    return None


def main(stepsPerEpoch=10, columnInReport=4, thresholdValue=2):
    return findFirstBindingEvent(stepsPerEpoch, columnInReport, thresholdValue)


if __name__ == "__main__":
    stepsForBE = main()
    print(stepsForBE)



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import time
import sys
import os
import numpy as np
import networkx as nx
from AdaptivePELE.clustering import clustering, thresholdcalculator
from AdaptivePELE.spawning import spawning, densitycalculator
# from AdaptivePELE.analysis import plot3DNetwork as net3D


def getUnvisitedPath(clustering, numClusters=5):
    conf = clustering.conformationNetwork
    visited = set()
    path = []
    stack = [0]
    while stack:
        node = stack.pop(0)
        path.append(node)
        visited.add(node)
        minVisit = 1e6
        minNeigh = None
        for _, target, data in conf.out_edges_iter(node, data=True):
            if data['transition'] < minVisit and target not in visited:
                minVisit = data['transition']
                minNeigh = target
        if minNeigh is not None:
            stack.append(minNeigh)

    if len(path) <= numClusters:
        return path
    else:
        return path[-1:-numClusters-1:-1]


def getShortestPath(clustering, numClusters=5):
    conf = clustering.conformationNetwork
    contacts = [cluster.contacts for cluster in clustering.clusterIterator()]
    maxCluster = np.argmax(contacts)
    shortPath = nx.shortest_path(conf, source=0, target=maxCluster, weight='transition')
    if len(shortPath) <= numClusters:
        return shortPath
    else:
        return shortPath[-1:-numClusters-1:-1]


def getMetastableClusters(clustering, numClusters=5):
    conf = clustering.conformationNetwork.network
    betweenness = nx.betweenness_centrality(conf, weight='transition')
    b2 = np.array([betweenness[i] for i in range(len(betweenness))])
    thresholds = [cluster.threshold for cluster in clustering.clusters.clusters]
    metInd = np.zeros_like(thresholds, dtype=np.float)
    for node in conf.nbunch_iter():
        if conf.degree(node) < 1:
            metInd[node] = 0.0
            continue
        totalIn = 0
        totalOut = 0
        selfTrans = 0
        totalKin = 0
        for source, _, data in conf.in_edges_iter(node, data=True):
            if source != node:
                totalIn += data['transition']*thresholds[source]**3
        for _, edge, data in conf.out_edges_iter(node, data=True):
            if edge == node:
                selfTrans = data['transition']
            totalOut += data['transition']*thresholds[edge]**3
            totalKin += data['transition']
        if totalOut+selfTrans:
            metInd[node] = (totalIn/float(totalOut))*(selfTrans/float(totalKin))
        else:
            metInd[node] = 0.0
    finalMetInd = metInd*b2
    return np.argsort(finalMetInd)[-1:-numClusters-1:-1]


def getMetastableClusters2(clustering, numClusters=5):
    conf = clustering.conformationNetwork.network
    betweenness = nx.betweenness_centrality(conf, weight='transition')
    b2 = np.array([betweenness[i] for i in range(len(betweenness))])
    return np.argsort(b2)[-1:-numClusters-1:-1]


def getMetastableClusters5(clustering, numClusters=5):
    conf = clustering.conformationNetwork.network
    thresholds = [cluster.threshold for cluster in clustering.clusters.clusters]
    minThres = min(thresholds)
    volume = np.zeros(len(thresholds))
    for node in conf.nbunch_iter():
        vol = 0.0
        for source, target in conf.edges_iter(node):
            assert source == node
            cluster = clustering.getCluster(target)
            vol += (cluster.threshold/minThres)**3
        volume[node] = vol
    return np.argsort(volume)[:numClusters+1]


def getMetastableClusters4(clustering, numClusters=5):
    conf = clustering.conformationNetwork.network
    metrics = [cluster.getMetricFromColumn(4) for cluster in clustering.clusters.clusters]
    indexes = np.zeros_like(metrics)
    outInd = np.zeros_like(metrics)
    inInd = np.zeros_like(metrics)
    with open("informationNetwork.csv", "w") as f:
        f.write("Node\tthreshold\tinDegree\toutDegree\tDegree\n")
        for node in conf.nbunch_iter():
            outNeigh = 0.0
            inNeigh = 0.0
            countOut = 0
            countIn = 0
            for _, target in conf.out_edges_iter(node):
                cluster = clustering.getCluster(target)
                countOut += 1
                outNeigh += (cluster.elements/cluster.threshold**3-outNeigh)/float(countOut)

            for source, _ in conf.in_edges_iter(node):
                cluster = clustering.getCluster(source)
                countIn += 1
                inNeigh += (cluster.elements/cluster.threshold**3-inNeigh)/float(countIn)
            outInd[node] = outNeigh
            inInd[node] = inNeigh
            if inNeigh > 1e-8:
                indexes[node] = outNeigh/inNeigh
            else:
                indexes[node] = 0.0
    return np.argsort(indexes)[-1:-numClusters-1:-1]


def getMetastableClusters3(clustering, numClusters=5):
    conf = clustering.conformationNetwork.network
    thresholds = [cluster.threshold for cluster in clustering.clusters.clusters]
    metInd = np.zeros_like(thresholds, dtype=np.float)
    for node in conf.nbunch_iter():
        if conf.degree(node) < 1:
            metInd[node] = 0.0
            continue
        totalIn = 0
        totalOut = 0
        selfTrans = 0
        totalKin = 0
        for source, _, data in conf.in_edges_iter(node, data=True):
            if source != node:
                totalIn += data['transition']*thresholds[source]**3
        for _, edge, data in conf.out_edges_iter(node, data=True):
            if edge == node:
                selfTrans = data['transition']
            totalOut += data['transition']*thresholds[edge]**3
            totalKin += data['transition']
        if totalOut+selfTrans:
            metInd[node] = (totalIn/float(totalOut))*(selfTrans/float(totalKin))
        else:
            metInd[node] = 0.0
    finalMetInd = metInd
    return np.argsort(finalMetInd)[-1:-numClusters-1:-1]

thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()
thresholdCalculator = thresholdCalculatorBuilder.build({
    "thresholdCalculator": {
        "type": "heaviside",
        "params": {
            "values": [2, 3, 4, 5],
            "conditions": [1, 0.75, 0.5]
        }
    }
})
# thresholdCalculator = thresholdCalculatorBuilder.build({})
# Distance index
# similarityEvaluator = clustering.CMSimilarityEvaluator("differenceDistance")
# thresholdCalculatorAcc = thresholdCalculatorBuilder.build({
#     "thresholdCalculator": {
#         "type": "heaviside",
#         "params": {
#             "conditions": [1.2, 1.0, 0.5, 0.0],
#             "values": [0.2, 0.4, 0.5, 0.8]
#         }
#     }
# })
# Jaccard index
similarityEvaluator = clustering.CMSimilarityEvaluator("Jaccard")
# thresholdCalculatorAcc = thresholdCalculatorBuilder.build({
#     "thresholdCalculator": {
#         "type" : "heaviside",
#         "params" : {
#             "conditions" : [1, 0.75, 0.5],
#             "values" : [0.025, 0.03, 0.04, 0.05]
#         }
#     }
# })
thresholdCalculatorAcc = thresholdCalculatorBuilder.build({
    "thresholdCalculator": {
        "type": "heaviside",
        "params": {
            "values": [0.2, 0.3, 0.5, 0.8],
            "conditions": [1.0, 0.75, 0.5]
        }
    }
})
# correlation
# similarityEvaluator = clustering.CMSimilarityEvaluator("correlation")
# thresholdCalculatorAcc = thresholdCalculatorBuilder.build({
#     "thresholdCalculator": {
#         "type": "constant",
#         "params": {
#             "value": 0.15
#         }
#     }
# })
densityCalculatorBuilder = densitycalculator.DensityCalculatorBuilder()
# densityCalculator = densityCalculatorBuilder.build({
#        "density": {
#             "type": "heaviside",
#             "params": {
#                  "conditions": [1.5, 1.0, 0.0],
#                  "values": [8.0, 2.37, 1.0, 0.5]
#              }
#         }
#    }
# )
densityCalculator = densityCalculatorBuilder.build({})
densityCalculator = densityCalculatorBuilder.build({
    "density": {
        "type": "continuous"
    }
})
spawnParams = spawning.SpawningParams()
spawnParams.buildSpawningParameters({
    "type": "inverselyProportional",
    "params": {
        "epsilon": 0.0,
        "T": 1000,
        "reportFilename": "report",
        "metricColumnInReport": 5
        }
    })
spawnParams.buildSpawningParameters({
    "type": "epsilon",
    "params": {
        "epsilon": 0.0,
        "T": 1000,
        "reportFilename": "report",
        "metricColumnInReport": 4
        }
    })
spawnParams.buildSpawningParameters({
    "type": "REAP",
    "params": {
        "epsilon": 0.0,
        "T": 1000,
        "reportFilename": "report",
        "metricsInd": -1,
        "metricColumnInReport": 4
        }
    })
contactThresholdDistance = 8
resname = "UI1"
nEpochs = 35
altSel = False
ntrajs = 144
ClCont = clustering.ContactsClustering(thresholdCalculator, resname=resname,
                                       reportBaseFilename="report",
                                       columnOfReportFile=4,
                                       contactThresholdDistance=contactThresholdDistance,
                                       symmetries=[], altSelection=altSel)
ClAcc = clustering.ContactMapAccumulativeClustering(thresholdCalculatorAcc,
                                                    similarityEvaluator,
                                                    resname=resname,
                                                    reportBaseFilename="report",
                                                    columnOfReportFile=4,
                                                    contactThresholdDistance=contactThresholdDistance,
                                                    altSelection=altSel)
# spawningObject = spawning.InverselyProportionalToPopulationCalculator(densityCalculator)
# spawningObject = spawning.UCBCalculator(densityCalculator)
# spawningObject = spawning.EpsilonDegeneracyCalculator(densityCalculator)
spawningObject = spawning.REAPCalculator()
# ClAcc.clusterInitialStructures(["/home/jgilaber/PR/PR_prog_initial_adaptive.pdb"])
# ClCont.clusterInitialStructures(["/home/jgilaber/4DAJ/4DAJ_initial_adaptive.pdb"])
# processorMapping = [0 for i in range(ntrajs-1)]
if not os.path.exists("mappings"):
    os.makedirs("mappings")
if not os.path.exists("results"):
    os.makedirs("results")
if not os.path.exists("networkEpochs"):
    os.makedirs("networkEpochs")
# fw = open("clusters.txt", "w")
# fw2 = open("clustersBet.txt", "w")
fw3 = open("clustersInd.txt", "w")
# fw4 = open("clustersIndNew.txt", "w")
# fw5 = open("clustersVol.txt", "w")
# fw6 = open("clustersVisit.txt", "w")
nModel = 1
for i in range(nEpochs):
    # path =["trajs/%d/run_traj*"%i]
    # paths_report = ["trajs/%d/run_report*"%i]
    # path = ["/home/bsc72/bsc72021/simulations/PR/testCM_4_32/simulation/PRprog_CM_variabExtra_UCB_5/%d/traj*" % i]
    # paths_report = ["/home/bsc72/bsc72021/simulations/PR/testCM_4_32/simulation/PRprog_CM_variabExtra_UCB_5//%d/report*" % i]
    # path = ["/gpfs/scratch/bsc72/bsc72021/AdaptiveCM/simulation/PRprog_4_64CMExtraSubset_prova_SASA3/%d/traj*" % i]
    # paths_report = ["/gpfs/scratch/bsc72/bsc72021/AdaptiveCM/simulation/PRprog_4_64CMExtraSubset_prova_SASA3/%d/report*" % i]
    # path = ["/home/jgilaber/PR/PR_simulation_network/%d/traj*" % i]
    # paths_report = ["/home/jgilaber/PR/PR_simulation_network/%d/report*" % i]
    path = ["/home/jgilaber/urokinases_free_energy/1sqa_adaptive_expl_sameR/%d/traj*" % i]
    paths_report = ["/home/jgilaber/urokinases_free_energy/1sqa_adaptive_expl_sameR/%d/report*" % i]
    trajs = clustering.getAllTrajectories(paths_report)
    total_snapshots = 0
    for traj in trajs:
        for line in open(traj, "r"):
            total_snapshots += 1
        total_snapshots -= 1
    sys.stderr.write("Total snapsthots for epoch %d: %d\n" % (i, total_snapshots))
    startTimeCont = time.time()
    # ClCont.cluster(path, processorMapping)
    ClCont.cluster(path)
    endTimeCont = time.time()
    sys.stderr.write("Total time of clustering contacts, epoch %d: %.6f\n" % (i, endTimeCont-startTimeCont))
    sys.stderr.write("Number of clusters contacts epoch %d: %d\n" % (i, len(ClCont.clusters.clusters)))
    # invTrajs = (ntrajs - 1)/2
    # centTrajs = ntrajs - 1- invTrajs
    invTrajs = ntrajs-1
    degeneraciesCont = spawningObject.calculate(ClCont.clusters.clusters, invTrajs, spawnParams)
    spawningObject.log()
    nProc = 0
    clusterList = []
    for icl in range(len(ClCont.clusters.clusters)):
        for j in range(int(degeneraciesCont[icl])):
            clusterList.append(ClCont.clusters.clusters[icl].trajPosition)
            nProc += 1
    assert nProc == ntrajs-1
    processorMapping = clusterList[1:]+[clusterList[0]]
    with open("%d/processorMapping.txt" % (i+1), "w") as f:
        f.write(':'.join(map(str, processorMapping)))
    # fNet = open("networkEpochs/network3d_%d.pdb" % nModel, "w")
    # metrics = [cl.getMetricFromColumn(4) for cl in ClCont.clusterIterator()]
    # Xn, Yn, Zn = net3D.getCoords(ClCont)
    # fNet.write("MODEL %d\n" % nModel)
    # fNet = net3D.writePDB(Xn, Yn, Zn, metrics, fNet)
    # fNet = net3D.writeConnectionsPDB(ClCont.conformationNetwork.network, fNet)
    # fNet.write("ENDMDL\n")
    # fNet.close()
    # nModel += 1
    ClCont.writeOutput("clsummary", degeneraciesCont, "ClCont.pkl", False)
    os.rename("clsummary/summary.txt", "results/summary_ClCont.txt")
    # sortedNodes = getMetastableClusters2(ClCont, centTrajs)
    # sortedNodes = set(sortedNodes).union(set([num for num, cl in enumerate(degeneraciesCont) if cl]))
    # print(sortedNodes)
    # for node in sortedNodes:
    #     cluster = ClCont.getCluster(node)
    #     fw3.write("%d\t%.3f\n" % (i*4, cluster.getMetricFromColumn(4)))

    # startTimeAcc = time.time()
    # ClAcc.cluster(path, processorMapping)
    # endTimeAcc = time.time()
    # sys.stderr.write("Total time of clustering accumulative, epoch %d: %.6f\n" % (i,endTimeAcc-startTimeAcc))
    # sys.stderr.write("Number of clusters accumulative epoch %d: %d\n" % (i,len(ClAcc.clusters.clusters)))
    # degeneraciesAcc = spawningObject.calculate(ClAcc.clusters.clusters, ntrajs-1, spawnParams)
    # # summaryFolder = "%d/clustering" % i
    # # if not os.path.exists(summaryFolder):
    # #     os.makedirs(summaryFolder)
    # nProc = 0
    # clusterList = processorMapping[:]
    # for icl in range(len(ClAcc.clusters.clusters)):
    #     for j in range(int(degeneraciesAcc[icl])):
    #         clusterList[nProc] = icl
    #         nProc += 1
    # assert nProc == ntrajs-1
    # ClAcc.writeOutput("clsummary", degeneraciesAcc, "ClAcc.pkl", False)
    # os.rename("clsummary/summary.txt", "results/summary_ClAcc.txt")
    # processorMapping = clusterList[1:]+[clusterList[0]]
    # with open("mappings/mapping%d.txt" % i, "w") as f:
    #     f.write(','.join(map(str, processorMapping)))
    # # sortedNodes = getMetastableClusters(ClAcc, 10)
    # # sortedNodes = set(sortedNodes)
    # # sortedNodes2 = getMetastableClusters2(ClAcc, 10)
    # # sortedNodes2 = set(sortedNodes2)
    # # sortedNodes3 = getMetastableClusters3(ClAcc, 10)
    # # sortedNodes3 = set(sortedNodes3)
    # # sortedNodes4 = getMetastableClusters4(ClAcc, 10)
    # # sortedNodes4 = set(sortedNodes4)
    # # sortedNodes5 = getMetastableClusters5(ClAcc, 10)
    # # sortedNodes5 = set(sortedNodes5)
    # # print(sortedNodes)
    # # print(sortedNodes2)
    # # print(sortedNodes3)
    # # print(sortedNodes4)
    # # print(sortedNodes5)
    # # for node in sortedNodes:
    # #     cluster = ClAcc.getCluster(node)
    # #     fw.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
    # # for node in sortedNodes2:
    # #     cluster = ClAcc.getCluster(node)
    # #     fw2.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
    # # for node in sortedNodes3:
    # #     cluster = ClAcc.getCluster(node)
    # #     fw3.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
    # # for node in sortedNodes4:
    # #     cluster = ClAcc.getCluster(node)
    # #     fw4.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
    # # for node in sortedNodes5:
    # #     cluster = ClAcc.getCluster(node)
    # #     fw5.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
    # # path = getUnvisitedPath(ClAcc, 10)
    # path = getShortestPath(ClAcc, 10)
    # print(path)
    # for node in path:
    #     cluster = ClAcc.getCluster(node)
    #     fw6.write("%d\t%.3f\n" % ((i+1)*4, cluster.originalMetrics[4]))
# fNet.close()



from __future__ import unicode_literals
import atomset
import numpy as np
cimport cython
cimport numpy as np
cimport frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.atomset as atomset


cdef class SymmetryContactMapEvaluator:
    def __init__(self, symmetries=[]):
        """
            :param symmetries: List of dictionaries with gropus of symmetric atoms atomId:symmetricalAtomId corresponding with the symmetrical atoms
            :type symmetries: list
        """
        self.symmetricAtoms = set()
        for group in symmetries:
            self.symmetricAtoms.update(set(group.keys()).union(set(group.values())))
        self.symmetries = symmetries
        self.symToRowMap = {}
        self.ligandList = []
        self.proteinList = []


    def __getstate__(self):
        state = {'symmetries': self.symmetries, 'symToRowMap': self.symToRowMap,
                 'symmetricAtoms': self.symmetricAtoms,
                 'ligandList': self.ligandList, 'proteinList': self.proteinList}
        return state

    def __setstate__(self, state):
        self.symmetries = state['symmetries']
        self.symToRowMap = state['symToRowMap']
        self.symmetricAtoms = state['symmetricAtoms']
        self.ligandList = state.get('ligandList', [])
        self.proteinList = state.get('proteinList', [])

    def createContactMap(self, atomset.PDB PDB, basestring ligandResname, int contactThresholdDistance, int ligandResnum=0, basestring ligandResChain=""):
        """
            Create the contact map of the protein and ligand. The contact map is
            a boolean matrix that has as many rows as the number of ligand heavy
            atoms and as many columns as the number of alpha carbons. The
            value is one if the ligand atom and the alpha carbons are less than
            contactThresholdDistance Amstrongs away

            :param ligandResname: Residue name of the ligand in the PDB
            :type ligandResname: basestring
            :param ligandResnum: Integer containing the residue number of the ligand in the pdb
            :type ligandResnum: int
            :param ligandResChain: String containing the chain name of the ligand in the pdb
            :type ligandResChain: basestring
            :param contactThresholdDistance: Maximum distance at which two atoms are considered in contanct (in Angstroms)
            :type contactThresholdDistance: int
            :returns: numpy.Array -- The contact map of the ligand and the protein
            :returns: int -- The number of alpha carbons in contact with the ligand
        """
        if self.symToRowMap:
            return self.buildContactMap(PDB, ligandResname, contactThresholdDistance, ligandResnum, ligandResChain)
        else:
            return self.buildContactMapWithRowMap(PDB, ligandResname, contactThresholdDistance, ligandResnum, ligandResChain)

    def buildContactMapWithRowMap(self, atomset.PDB PDBobj, basestring ligandResname, int contactThresholdDistance, int ligandResnum=0, basestring ligandResChain=""):
        """
            Create a map that relates symmetric atoms in the ligand to row
            indices in the contactMap matrix and the contact map of the protein
            and ligand. The contact map is a boolean matrix that has as many
            rows as the number of ligand heavy atoms and as many columns as the
            number of alpha carbons. The value is one if the ligand atom and
            the alpha carbons are less than contactThresholdDistance Amstrongs
            away

            :param PDBobj: PDB object that contains the conformation's info
            :type PDBobj: PDB
            :param ligandResname: Residue name of the ligand in the PDB
            :type ligandResname: basestring
            :param ligandResnum: Integer containing the residue number of the ligand in the pdb
            :type ligandResnum: int
            :param ligandResChain: String containing the chain name of the ligand in the pdb
            :type ligandResChain: basestring
            :param contactThresholdDistance: Maximum distance at which two atoms are considered in contanct (in Angstroms)
            :type contactThresholdDistance: int
            :returns: numpy.Array -- The contact map of the ligand and the protein
            :returns: int -- The number of alpha carbons in contact with the ligand
        """
        cdef int contactThresholdDistance2, rowind, colind
        contactThresholdDistance2 = contactThresholdDistance**2
        cdef atomset.PDB ligandPDB, alphaCarbonsPDB
        cdef np.ndarray[char, ndim=2] contactMap
        cdef basestring ligandAtomID, proteinAtomID
        cdef atomset.Atom proteinAtom, ligandAtom
        cdef set contacts
        cdef double dist2
        ligandPDB = atomset.PDB()
        ligandPDB.initialise(PDBobj.pdb, resname=ligandResname, resnum=ligandResnum,
                             chain=ligandResChain, heavyAtoms=True)
        alphaCarbonsPDB = atomset.PDB()
        alphaCarbonsPDB.initialise(PDBobj.pdb, type="CM")
        if not self.ligandList:
            self.ligandList = ligandPDB.atomList
        if not self.proteinList:
            self.proteinList = alphaCarbonsPDB.atomList
        # empty contact map, rows are atoms of the ligand, columns are protein
        # alpha carbons
        contactMap = np.zeros((len(ligandPDB.atomList),
                               len(alphaCarbonsPDB.atomList)), dtype=np.uint8)
        contacts = set([])
        for rowind in range(len(ligandPDB.atomList)):
            ligandAtomID = ligandPDB.atomList[rowind]
            ligandAtom = ligandPDB.atoms[ligandAtomID]
            if ligandAtomID in self.symmetricAtoms:
                self.symToRowMap[ligandAtomID] = rowind
            for colind in range(len(alphaCarbonsPDB.atomList)):
                proteinAtomID = alphaCarbonsPDB.atomList[colind]
                proteinAtom = alphaCarbonsPDB.atoms[proteinAtomID]
                dist2 = ligandAtom.squaredDistance(proteinAtom)
                if dist2 < contactThresholdDistance2:
                    contactMap[rowind, colind] = True
                if proteinAtom.name == "CA" and dist2 < 64.0:
                    # Contact ratio will be always calculated using a contact
                    # threshold of 8, so that tresholds and densities are
                    # independent of the contact threshold of the contactMap
                    contacts.update([proteinAtomID])
        return contactMap.view(np.bool), len(contacts)

    def buildContactMap(self, atomset.PDB PDBobj, basestring ligandResname, int contactThresholdDistance, int ligandResnum=0, basestring ligandResChain=""):
        """
            Create the contact map of the protein and ligand. The contact map is
            a boolean matrix that has as many rows as the number of ligand heavy
            atoms and as many columns as the number of alpha carbons. The
            value is one if the ligand atom and the alpha carbons are less than
            contactThresholdDistance Amstrongs away

            :param PDBobj: PDB object that contains the conformation's info
            :type PDBobj: PDB
            :param ligandResname: Residue name of the ligand in the PDB
            :type ligandResname: basestring
            :param ligandResnum: Integer containing the residue number of the ligand in the pdb
            :type ligandResnum: int
            :param ligandResChain: String containing the chain name of the ligand in the pdb
            :type ligandResChain: basestring
            :param contactThresholdDistance: Maximum distance at which two atoms are considered in contanct (in Angstroms)
            :type contactThresholdDistance: int
            :returns: numpy.Array -- The contact map of the ligand and the protein
            :returns: int -- The number of alpha carbons in contact with the ligand
        """
        cdef int contactThresholdDistance2, rowind, colind
        contactThresholdDistance2 = contactThresholdDistance**2
        cdef atomset.PDB ligandPDB, alphaCarbonsPDB
        cdef np.ndarray[char, ndim=2] contactMap
        cdef basestring ligandAtomID, proteinAtomID
        cdef atomset.Atom proteinAtom, ligandAtom
        cdef set contacts
        cdef double dist2
        ligandPDB = atomset.PDB()
        ligandPDB.initialise(PDBobj.pdb, resname=ligandResname, resnum=ligandResnum,
                             chain=ligandResChain, heavyAtoms=True)
        alphaCarbonsPDB = atomset.PDB()
        alphaCarbonsPDB.initialise(PDBobj.pdb, type="CM")
        if not self.ligandList:
            self.ligandList = ligandPDB.atomList
        if not self.proteinList:
            self.proteinList = alphaCarbonsPDB.atomList
        # empty contact map, rows are atoms of the ligand, columns are protein
        # alpha carbons
        contactMap = np.zeros((len(ligandPDB.atomList),
                               len(alphaCarbonsPDB.atomList)), dtype=np.uint8)
        contacts = set([])
        for rowind in range(len(ligandPDB.atomList)):
            ligandAtomID = ligandPDB.atomList[rowind]
            ligandAtom = ligandPDB.atoms[ligandAtomID]
            if ligandAtomID in self.symmetricAtoms:
                self.symToRowMap[ligandAtomID] = rowind
            for colind in range(len(alphaCarbonsPDB.atomList)):
                proteinAtomID = alphaCarbonsPDB.atomList[colind]
                proteinAtom = alphaCarbonsPDB.atoms[proteinAtomID]
                dist2 = ligandAtom.squaredDistance(proteinAtom)
                if dist2 < contactThresholdDistance2:
                    contactMap[rowind, colind] = True
                if proteinAtom.name == "CA" and dist2 < 64.0:
                    # Contact ratio will be always calculated using a contact
                    # threshold of 8, so that tresholds and denisities are
                    # independent of the contact threshold of the contactMap
                    contacts.update([proteinAtomID])
        return contactMap.view(np.bool), len(contacts)

    def evaluateJaccard(self, contactMap, clusterContactMap):
        permContactMap = self.buildOptimalPermutationContactMap(contactMap,
                                                                clusterContactMap)
        intersectContactMaps = (permContactMap & clusterContactMap).sum()
        unionContactMaps = permContactMap.sum() + clusterContactMap.sum() - intersectContactMaps
        if unionContactMaps < 1e-7:
            # both contactMaps have zero contacts
            return 0.0
        similarity = float(intersectContactMaps)/unionContactMaps
        distance = 1-similarity
        return distance

    def evaluateCorrelation(self, contactMap, clusterContactMap):
        permContactMap = self.buildOptimalPermutationContactMap(contactMap,
                                                                clusterContactMap)
        similarity = calculateCorrelationContactMaps(permContactMap, clusterContactMap)
        similarity += 1  # Necessary to omit negative correlations
        similarity /= 2.0  # Correlation values need to be higher now
        distance = 1-similarity
        return distance

    def evaluateDifferenceDistance(self, contactMap, clusterContactMap):
        permContactMap = self.buildOptimalPermutationContactMap(contactMap,
                                                                clusterContactMap)
        differenceContactMaps = np.abs(permContactMap^clusterContactMap).sum()
        averageContacts = (0.5*(permContactMap.sum()+clusterContactMap.sum()))
        if not averageContacts:
            # The only way the denominator can be 0 is if both contactMaps are
            # all zeros, thus being equal and belonging to the same cluster
            return 0.0
        else:
            distance = differenceContactMaps/averageContacts
            return distance

    def buildOptimalPermutationContactMap(self, contactMap, clusterContactMap):
        """
            Build a permutated version of the contactMap which maximizes the
            similarity between the two contactMaps
            :param contactMap: contactMap of the conformation to compare
            :type contactMap: numpy.Array
            :param clusterContactMap: cluster contactMap to which we are comparing
            :type cluster: numpy.Array
            :returns: numpy.Array -- The permutated contact map
        """
        if not self.symmetries and not self.symToRowMap:
            return contactMap
        permContactMap = np.array(contactMap)
        for group in self.symmetries:
            for atom1Id, atom2Id in group.iteritems():
                try:
                    atom1Row = self.symToRowMap[atom1Id]
                    atom2Row = self.symToRowMap[atom2Id]
                    line11 = clusterContactMap[atom1Row]
                    line12 = clusterContactMap[atom2Row]
                    line21 = contactMap[atom1Row]
                    line22 = contactMap[atom2Row]
                except KeyError as err:
                    raise KeyError("Atom %s not found in symmetries" % err.message)
                d2 = (line11 == line21).sum() + (line12 == line22).sum()
                d2sm = (line12 == line21).sum() + (line11 == line22).sum()
                if d2sm > d2:
                    # swap rows using advanced slicing from numpy
                    permContactMap[[atom1Row, atom2Row], :] = contactMap[[atom2Row, atom1Row], :]
        return permContactMap


def calculateCorrelationContactMaps(contactMap, clusterContactMap):
    """
        Calculate the correlation of two contactMaps
    """
    contactMap1 = contactMap.reshape((1, -1))
    # Reshape the matrix into a 1D array to use the numpy corrcoef function
    contactMap2 = clusterContactMap.reshape((1, -1))
    total1 = contactMap1.sum()
    total2 = contactMap2.sum()
    if not total1 or not total2:
        # if any array is all zeros the correlation will be NaN
        # if both are zero, correlation is perfect (i.e 1) else it is different
        # and will be in different clusters
        return total1 == total2
    return np.corrcoef(contactMap1, contactMap2)[0, 1]



from __future__ import absolute_import, division, print_function, unicode_literals
import glob
import os
from AdaptivePELE.freeEnergies import estimateDG
import numpy as np


def main(trajsPerEpoch, lagtime, nclusters, clusteringStride=1, nruns=10):
    allFolders = np.array(glob.glob("MSM_*"))
    epochs = [int(folder[4:]) for folder in allFolders]
    args = np.argsort(epochs)
    sortedFolders = allFolders[args]
    origDir = os.getcwd()
    resultsFile = os.path.join(origDir, "results.txt")

    with open(resultsFile, "a") as f:
        f.write("#Epoch DG StdDG Db StdDb\n")
        f.write("#=======================\n")

    resultsEpoch = []
    initialEpoch = 0
    for i, folder in enumerate(sortedFolders[initialEpoch:]):
        epoch = i + initialEpoch
        print(epoch, folder)
        os.chdir(folder)
        parameters = estimateDG.Parameters(ntrajs=trajsPerEpoch*(epoch+1),
                                           length=None,
                                           lagtime=lagtime,
                                           lagtimes=[1, 10, 25],
                                           nclusters=nclusters,
                                           nruns=nruns,
                                           useAllTrajInFirstRun=True,
                                           computeDetailedBalance=True,
                                           trajWildcard="traj_*",
                                           folderWithTraj="rawData",
                                           clusterCountsThreshold=0,
                                           clusteringStride=clusteringStride)
        dG, stdDg, db, stdDb = estimateDG.estimateDG(parameters, cleanupClusterCentersAtStart=True)
        print("FINAL RESULTS EPOCH %d: dG: %f +- %f, asymmetric fluxes: %f +- %f" % (epoch, dG, stdDg, db, stdDb))
        resultsEpoch.append([dG, stdDg, db, stdDb])
        with open(resultsFile, "a") as f:
            f.write("%d %.3f %.3f %.3f %.3f\n" % (epoch, dG, stdDg, db, stdDb))
        os.chdir("..")

    print("Results")
    print("epoch, DG, stdDg, DB, stdDb")
    print("=====")
    for i, results in enumerate(resultsEpoch):
        print(i, results)

if __name__ == "__main__":
    trajs_Epoch = 50
    lag_time = 50
    n_clusters = 100
    clustering_Stride = 10
    main(trajs_Epoch, lag_time, n_clusters, clusteringStride=clustering_Stride)



"""
    Adds rejected steps in report

    To be run in simulation root folder

    Needs refactoring
"""
from __future__ import absolute_import, division, print_function, unicode_literals
import os
import glob

# to be changed by user
reportFilename = "report_"
reportOutputFilename = "allReport_"
stepsPerEpoch = 4

allFolders = os.listdir(".")
epochFolders = [epoch for epoch in allFolders if epoch.isdigit()]

for folder in epochFolders:
    print("Writing reports in folder", folder)
    os.chdir(folder)

    reports = glob.glob(reportFilename + "*")
    for report in reports:
        with open(report, "r") as f:
            newReportContent = []
            reportContent = f.readlines()

        newReportContent.append(reportContent[0])
        count = 0
        for i in range(1, len(reportContent)-1):  # the last one will be checked against steps per epoch
            splitContent = reportContent[i].split()
            thisAccStep = int(float(splitContent[1]))
            nextAccStep = int(float(reportContent[i+1].split()[1]))
            for j in range(nextAccStep-thisAccStep):
                splitContent[1] = str(count)
                newReportContent.append(" ".join(splitContent))
                count += 1

        # last one is always the steps per epoch (+1, since 0 is initial)
        splitContent = reportContent[-1].split()
        lastAccStep = int(float(splitContent[1]))
        lastTotalStep = stepsPerEpoch + 1
        for i in range(lastTotalStep - lastAccStep):
            splitContent[1] = str(count)
            newReportContent.append(" ".join(splitContent))
            count += 1

        # write file
        reportNum = report.split("_")[-1]
        with open(reportOutputFilename + reportNum, "w") as f:
            for line in newReportContent:
                f.write(line + "\n")

    os.chdir("..")



from __future__ import absolute_import, division, print_function, unicode_literals
import pickle
import numpy as np
import matplotlib.pyplot as plt
from AdaptivePELE.atomset import atomset
from mpl_toolkits.mplot3d import Axes3D
import argparse


def extractCOMMatrix(clusters, resname):
    """ Extract a matrix contaning the coordinates of the center of mass of
    the ligand for each cluster structure

        clusters [In] List of clusters
        resname [In] Residue name of the ligand in the pdb
    """
    n = len(clusters)
    cluster_matrix = np.zeros((n, 3))
    metrics = np.zeros(n)
    population = np.zeros(n)
    total_elements = 0
    contacts = np.zeros(n)
    for index, cluster in enumerate(clusters):
        metrics[index] = cluster.metrics[cluster.metricCol]
        contacts[index] = cluster.contacts
        ligandPDB = atomset.PDB()
        ligandPDB.initialise(cluster.pdb.pdb, resname=resname)
        cluster_matrix[index, :] = ligandPDB.extractCOM()
        population[index] = cluster.elements
        total_elements += cluster.elements
    return cluster_matrix, metrics, total_elements, population, contacts


def plotClusters2D(cluster_matrix, metrics, title):
    """ Create all combination of xyz projections in 2D of the scatter plot
    of the center of mass of the ligand with a colormap given by a certain
    quantity (usually a metric or the clusters population)

        cluster_matrix [In] matrix contaning the coordinates of the center of
        mass of the ligand for each cluster structure
        metrics [In] Array with the quantity that will be used to create the
        colormap
        title [In] Title for the plot figure
    """
    ccx = cluster_matrix[:, 0]
    ccy = cluster_matrix[:, 1]
    ccz = cluster_matrix[:, 2]
    fig, axes = plt.subplots(nrows=2, ncols=2, sharex='col',
                             sharey='row')
    fig.suptitle(title)
    scatter1 = axes[0][0].scatter(ccx, ccy, c=metrics)  # ,label="Set %d"%index)
    axes[0][1].scatter(ccz, ccy, c=metrics)  # ,label="Set %d"%index)
    axes[1][0].scatter(ccx, ccz, c=metrics)  # ,label="Set %d"%index)
    fig.colorbar(scatter1)

#    axes[1][0].legend(loc='center right', bbox_to_anchor=[1.8,0.5])
    axes[0][0].set_ylabel('y')
    axes[1][0].set_ylabel('z')
    axes[1][0].set_xlabel('x')
    axes[1][1].axis('off')
    axes[0][1].set_xticks(axes[1][1].get_xticks())
    axes[0][1].set_xticklabels(axes[1][1].get_xticklabels())
    axes[0][1].set_xlabel('z')
    return fig


def plotClusters(cluster_matrix, metrics, title):
    """ Create a 3D scatter plot of the center of mass of the ligand with a
    colormap given by a certain quantity
    (usually a metric or the clusters population)

        cluster_matrix [In] matrix contaning the coordinates of the center of
        mass of the ligand for each cluster structure
        metrics [In] Array with the quantity that will be used to create the
        colormap
        title [In] Title for the plot figure
    """
    fig = plt.figure()
    ax = Axes3D(fig)
    ccx = cluster_matrix[:, 0]
    ccy = cluster_matrix[:, 1]
    ccz = cluster_matrix[:, 2]
    fig.suptitle(title)
    scatter1 = ax.scatter(ccx, ccy, zs=ccz, c=metrics)
    fig.colorbar(scatter1)
    ax.set_ylabel('y')
    ax.set_zlabel('z')
    ax.set_xlabel('x')
    return fig


def extractInfo(inputFile):
    clusterInfo = np.loadtxt(inputFile)
    return clusterInfo[:, 1]


def plotClusteringData(pklObjectFilename, resname, titlemetric, titlepopulation,
                       titlecontacts, metricPlotFilename="",
                       populationPlotFilename="", contactsPlotFilename="",
                       metricFlag=False, populationFlag=False,
                       contactsFlag=False, inputFile=None):

    with open(pklObjectFilename, "r") as f:
        clObject = pickle.load(f)

    comCoord, metrics, totalElements, population, contacts = extractCOMMatrix(clObject.clusters.clusters, resname)

    if inputFile is not None:
        clustersInfo = extractInfo(inputFile)
        plotClusters(comCoord, clustersInfo, "")

    if metricFlag:
        plot = plotClusters(comCoord, metrics, titlemetric)
        if metricPlotFilename:
            plot.savefig(metricPlotFilename)

    if populationFlag:
        plotContpop = plotClusters(comCoord, population, titlepopulation)
        if populationPlotFilename:
            plotContpop.savefig(populationPlotFilename)

    if contactsFlag:
        plotContcont = plotClusters(comCoord, contacts, titlecontacts)
        if contactsPlotFilename:
            plotContcont.savefig(contactsPlotFilename)

    print("Number of elements", totalElements)


def parseArguments():
    desc = "3D visualization of clustering"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("clusteringObject", type=str, help="Clustering object")
    parser.add_argument("resname", type=str, help="Resname in the pdb")
    parser.add_argument("-metrics", action="store_true", help="Wether to plot the metric of the clusters as color")
    parser.add_argument("-population", action="store_true", help="Wether to plot the population of the clusters as color")
    parser.add_argument("-contacts", action="store_true", help="Wether to plot the contacts of the clusters as color")
    parser.add_argument("-i", type=str, default=None, help="File with cluster-associated information")
    args = parser.parse_args()

    return args.clusteringObject, args.resname, args.metrics, args.population, args.contacts, args.i

if __name__ == "__main__":
    pklObject_filename, lig_resname, metricsFlag, population_flag, contacts_flag, input_file = parseArguments()

    metricPlot_filename = ""  # "results/contactClusters.png"
    populationPlot_filename = ""  # "results/contactClusterspop.png"
    contactsPlot_filename = ""  # "results/contactClustersContacts.png"
    title_metric = "Metrics Contacts"
    title_population = "Population Contacts"
    title_contacts = "Number of contacts Contacts"

    plotClusteringData(pklObject_filename, lig_resname, title_metric, title_population,
                       title_contacts, metricPlot_filename,
                       populationPlot_filename, contactsPlot_filename, metricsFlag,
                       population_flag, contacts_flag, input_file)
    plt.show()
